/**
 * Game of Life
 * 
 */
package gridlife.core;


import gridlife.ui.GridLifeGuiConnector;



/**
 * Primitive animation thread causing a continuous computation of follow-up generations. 
 * 
 * @author Ulf Wagemann
 */
public class GridLifeAnimationThread  extends Thread
{
 private GridLifeGuiConnector m_tGuiConnector;
 private boolean              m_bRunning     ;
 private int                  m_iDelay       ;
 
 
 /**
  * Constructor
  * 
  * @param pa_tGuiConnector   hook to main class
  * @param pa_iDelay          animation delay in ms
  */
 public GridLifeAnimationThread(GridLifeGuiConnector pa_tGuiConnector, int pa_iDelay)
 {
  super();  
  
  m_iDelay        = pa_iDelay;
  m_bRunning      = true;
  m_tGuiConnector = pa_tGuiConnector;
 }
 
 
 
 /**
  * Main animation loop
  * 
  */
 @Override
 public void run()
 {
  while (true == m_bRunning)
  {
   if (null != m_tGuiConnector)
   {
    m_tGuiConnector.computeNextGeneration(); 
   }

   try
   {
    sleep(m_iDelay); 
   }
   catch(InterruptedException lc_tException)
   {}
  } // while
 }
 
 
 
 /**
  * Sets the animation delay to the given value
  * 
  * @param pa_iDelay
  */
 public final void setAnimationDelay (int pa_iDelay)
 {
  if (0 < pa_iDelay && Integer.MAX_VALUE > pa_iDelay)
    m_iDelay = pa_iDelay;
 }
 
 
 
 /**
  * Softly stops the animation main loop
  * 
  */
 public final void stopAnimation()
 {
  m_bRunning = false; 
 }
}
