/*
 * Game of Life
 * 
 */
package gridlife.ui;



/**
 * Imports
 * 
 */
import gridlife.core.GridLifeConstants    ;
import gridlife.color.GridLifeColorManager;
import gridlife.core.GridLifeProperties   ;
import gridlife.shape.GridLifeShape       ;

import java.awt.Color         ;
import java.awt.Point         ;
import java.awt.Rectangle     ;
import java.awt.Graphics2D    ;
import java.awt.Composite     ;
import javax.swing.JScrollPane;



/**
 * Main cell drawing panel.
 * 
 * @author Ulf Wagemann
 */
public class GridLifeCellMatrixPaintPanel extends GridLifeCellPaintPanel
{
 private Point  m_tPaletteEditorShapePos      ;
 private Point  m_tPaletteShapePos            ;
 private Point  m_tSelectionShapePos          ;
 private Point  m_tFramePos                   ;
 private Point  m_tCellSelectionPosTopLeft    ;
 private Point  m_tCellSelectionPosBottomRight;

 private boolean m_bHasPaletteShape      ;
 private boolean m_bHasSelectionShape    ;
 private boolean m_bHasCellSelection     ;
 private boolean m_bHasPaletteEditorShape;
 private boolean m_bScreenshotMode       ;
 
 private boolean  m_bPaintFrame             ;
 private boolean  m_bPaintPaletteShape      ;
 private boolean  m_bPaintPaletteEditorShape;
 private boolean  m_bPaintSelectionShape    ;
 private boolean  m_bPaintSelectionFrame    ;
 private boolean  m_bPaintShapeModeXor      ;
        
 private int m_iShowHelpLines;
 private int m_iShowGridLines;
 private int m_iShowCellHiliteLines;
 private int m_iShowCellBorders;
 
 private GridLifeShape        m_tPaletteEditorShape;
 private GridLifeShape        m_tPaletteShape      ;
 private GridLifeShape        m_tSelectionShape    ;
 
 private GridLifeColorManager m_tColorManager      ;
 private Color                m_tCurrentCellFillColor;
 
 
 /**
   * Constructor
   * 
   */
 public GridLifeCellMatrixPaintPanel(int[][] pa_tCells, JScrollPane pa_tScrollPane, GridLifeProperties pa_tProperties, 
                                     GridLifeColorManager pa_tColorManager, int pa_iCellSize)
 {
  super(pa_tCells, pa_tScrollPane, pa_tProperties, pa_iCellSize);

  m_bPaintShapeModeXor       = false;
  m_bHasPaletteShape         = false;
  m_bPaintPaletteShape       = false;
  m_bPaintFrame              = false;
  m_bPaintSelectionFrame     = false;
  m_bHasCellSelection        = false;
  m_bPaintSelectionShape     = false;
  m_bHasSelectionShape       = false;
  m_bPaintPaletteEditorShape = false;
  m_bHasPaletteEditorShape   = false;
  m_bScreenshotMode          = false;
  
  m_tSelectionShape     = null;
  m_tPaletteShape       = null;
  m_tPaletteEditorShape = null;
  
  m_iShowCellHiliteLines = 0;
  m_iShowGridLines       = 0;
  m_iShowHelpLines       = 0;
  m_iShowCellBorders     = 0;
  
  m_tPaletteEditorShapePos       = new Point(1,1);
  m_tSelectionShapePos           = new Point(1,1);
  m_tPaletteShapePos             = new Point(1,1);
  m_tFramePos                    = new Point(1,1);  
  m_tCellSelectionPosTopLeft     = new Point(1,1);
  m_tCellSelectionPosBottomRight = new Point(1,1);
  
  m_tColorManager        = pa_tColorManager;
  m_iShowHelpLines       = pa_tProperties.getShowHelpLines();
  m_iShowGridLines       = pa_tProperties.getShowGridLines();
  m_iShowCellHiliteLines = pa_tProperties.getShowCellHiliteLines();
  
  if (null != pa_tProperties)
  {
   m_iShowCellHiliteLines = pa_tProperties.getShowCellHiliteLines()  ;
   m_iShowGridLines       = pa_tProperties.getShowGridLines()        ;
   m_iShowHelpLines       = pa_tProperties.getShowHelpLines()        ;
   m_iShowCellBorders     = pa_tProperties.getShowCellBorders()      ; 
   m_tColorBackground     = pa_tProperties.getColorBg()              ; 
   m_tGridHelpLineColor   = pa_tProperties.getColorHelpLines()       ;
   m_tCellHiliteColor     = pa_tProperties.getColorCellHilightLines();
   m_tGridColor           = pa_tProperties.getColorGridLines()       ;
   
   m_tCellMatrixSelectionFrameColor  = pa_tProperties.getColorCellSelectionFrame();
   m_tCellMatrixSelectionAreaColor    = pa_tProperties.getColorCellSelectionArea();
   
   setBackground(m_tColorBackground);
  }
  
  m_tCurrentCellFillColor = new Color(0xCCCCCC);
 }
 

 /**
  * Returns the screenshode mode
  * 
  * @return  true, if screenshot mode is active, false otherwise
  */
 public final boolean getScreenshotMode() {return m_bScreenshotMode;}
 
 
 
 /**
  * Setsthe screenshode mode
  * 
  * @param pa_bValue  true, if screenshot mode is to be active, false otherwise
  */
 public final void setScreenshotMode(boolean pa_bValue) {m_bScreenshotMode = pa_bValue;}
 
 
 
 /**
  * Sets the background color
  * 
  * @param pa_tColor   new background color
  */
 public final void setColorBg(Color pa_tColor) 
 {
  if (null != pa_tColor)
  {
   m_tColorBackground = new Color(pa_tColor.getRGB());
   setBackground(m_tColorBackground);
  }
 }
 
 
  /**
  * Sets the cell border color
  * 
  * @param pa_tColor   new cell border color
  */
 public final void setColorCellBorders(Color pa_tColor) 
 {
  if (null != pa_tColor)
  {
   m_tCellBorderColor = new Color(pa_tColor.getRGB());
  }
 }
 
 
 
 /**
  * Sets the cell selection frame color
  * 
  * @param pa_tColor   new  cell selection frame color
  */
 public final void setColorCellSelectionFrame(Color pa_tColor) 
 {
  if (null != pa_tColor)
  {
   m_tCellMatrixSelectionFrameColor = new Color(pa_tColor.getRGB());
  }
 } 

 
 
 /**
  * Sets the cell selection frmae color
  * 
  * @param pa_tColor   new  cell selection area color
  */
 public final void setColorCellSelectionArea(Color pa_tColor) 
 {
  if (null != pa_tColor)
  {
   m_tCellMatrixSelectionAreaColor = new Color(pa_tColor.getRGB());
  }
 } 

 
 
 /**
  * Sets the cell selection frame color
  * 
  * @param pa_tColor   new  cell selection frame color
  */
 public final void setColorCellInsertionFrame(Color pa_tColor) 
 {
  if (null != pa_tColor)
  {
   m_tCellMatrixInsertionFrameColor = new Color(pa_tColor.getRGB());
  }
 } 

 
 
 /**
  * Sets the cell selection frmae color
  * 
  * @param pa_tColor   new  cell selection area color
  */
 public final void setColorCellInsertionArea(Color pa_tColor) 
 {
  if (null != pa_tColor)
  {
   m_tCellMatrixInsertionAreaColor = new Color(pa_tColor.getRGB());
  }
 } 
 
 
 
 /**
  * Sets the grdlines color
  * 
  * @param pa_tColor   new background color
  */
 public final void setColorGridLines(Color pa_tColor) 
 {
  if (null != pa_tColor)
  {
   m_tGridColor = new Color(pa_tColor.getRGB());
  }
 }
 

 
 /**
  * Sets the grdlines color
  * 
  * @param pa_tColor   new background color
  */
 public final void setColorHelpLines(Color pa_tColor) 
 {
  if (null != pa_tColor)
  {
   m_tGridHelpLineColor = new Color(pa_tColor.getRGB());
  }
 }


 
 /**
  * Sets the grdlines color
  * 
  * @param pa_tColor   new background color
  */
 public final void setColorCellHilightLines(Color pa_tColor) 
 {
  if (null != pa_tColor)
  {
   m_tCellHiliteColor = new Color(pa_tColor.getRGB());
  }
 }

 
 
 /**
  * Defines the helplines shown
  * 
  * @param pa_iValue  0 = none, 1 = 50%, 2 = all
  */
 public final void setShowHelpLines(int pa_iValue)
 {
  if (0 <= pa_iValue && 3 > pa_iValue)  
    m_iShowHelpLines = pa_iValue;
 }
 
 
/**
  * Defines the gridlines
  * 
  * @param pa_iValue  0 = no, 1 = yes
  */
 public final void setShowGridLines(int pa_iValue)
 {
  if (0 <= pa_iValue && 2 > pa_iValue)  
    m_iShowGridLines = pa_iValue;
 }
 

  /**
  * Defines the cell hilite lines
  * 
  * @param pa_iValue  0 = no, 1 = yes
  */
 public final void setShowCellHiliteLines(int pa_iValue)
 {
  if (0 <= pa_iValue && 2 > pa_iValue)  
    m_iShowCellHiliteLines = pa_iValue;
 }
 
 

  /**
  * Defines the cell hilite lines
  * 
  * @param pa_iValue  0 = no, 1 = yes
  */
 public final void setShowCellBorders(int pa_iValue)
 {
  if (0 <= pa_iValue && 2 > pa_iValue)  
    m_iShowCellBorders = pa_iValue;
 }
 
 
 
 /**
  * Sets the fill color zo the givrn color
  * 
  * @param pa_tColor    new fill color
  */
 public final void setCellFillColor(Color pa_tColor)
 {
  m_tCurrentCellFillColor = (null != pa_tColor ? pa_tColor : new Color(0xCCCCCC)); 
 }
 
 
 
 /**
  * Sets the XOR painting mode to the given mode
  * 
  * @param pa_bMode mode
  */
 public final void setShapeXorPaintMode(boolean pa_bMode)
 {
  m_bPaintShapeModeXor = pa_bMode; 
 }
 
 
 
 /**
  * Returns the current palette shape (might have been rotated or mirrored by the user)
  * 
  * @return  shape, null in case of error
  */
 public final GridLifeShape getPaletteShape() {return m_tPaletteShape;}
 
 
 
 
 /**
  * Returns the current palette shape (might have been rotated or mirrored by the user)
  * 
  * @return  shape, null in case of error
  */
 public final GridLifeShape getPaletteEditorShape() {return m_tPaletteEditorShape;}
 
 
 
 /**
  * Returns the current palette shape (might have been rotated or mirrored by the user)
  * 
  * @return  shape, null in case of error
  */
 public final GridLifeShape getSelectionShape() {return m_tSelectionShape;}
 
 
  
 /**
  * Rotates the current palette shape 90 degrees clockwise
  * 
  */
 public final void rotatePaletteEditorShape()
 {
  if (null != m_tPaletteEditorShape) 
  {
   m_tPaletteEditorShape.rotate();
   repaint();
  }
 }
 
 
 /**
  * Rotates the current palette shape 90 degrees clockwise
  * 
  */
 public final void mirrorPaletteEditorShape()
 {
  if (null != m_tPaletteEditorShape) 
  {
   m_tPaletteEditorShape.mirror();
   repaint();
  }
 }
 
 
 
 /**
  * Rotates the current palette shape 90 degrees clockwise
  * 
  */
 public final void rotatePaletteShape()
 {
  if (null != m_tPaletteShape) 
  {
   m_tPaletteShape.rotate();
   repaint();
  }
 }
 
 
 /**
  * Rotates the current palette shape 90 degrees clockwise
  * 
  */
 public final void mirrorPaletteShape()
 {
  if (null != m_tPaletteShape) 
  {
   m_tPaletteShape.mirror();
   repaint();
  }
 }
 
 
 
 /**
  * Rotates the current selection shape 90 degrees clockwise
  * 
  */
 public final void rotateSelectionShape()
 {
  if (null != m_tSelectionShape) 
  {
   m_tSelectionShape.rotate();
   repaint();
  }
 }
 
 
 
 /**
  * Rotates the current selection shape 90 degrees clockwise
  * 
  */
 public final void mirrorSelectionShape()
 {
  if (null != m_tSelectionShape) 
  {
   m_tSelectionShape.mirror();
   repaint();
  }
 }
 
 
 
 /**
  * Does the custom painting on the given graphics object.
  * 
  * @param pa_tGraphics  Paint on it!
  */
 public void paintCustom(Graphics2D pa_tGraphics)
 {
  Rectangle  lc_tRect       = null;
  
  //
  // get visible rect for later comparisions
  //
  if (null != m_tScrollPane)
  {
   if (null != m_tScrollPane.getViewport())
   {
    lc_tRect = m_tScrollPane.getViewport().getViewRect();
   }
  }
  
  //
  // only continue with valid objects
  //
  if (null != pa_tGraphics && null != lc_tRect)
  { 
   pa_tGraphics.setRenderingHints(m_tRenderingHints);
   
   //
   // draw grid
   //
   drawGrid(pa_tGraphics, lc_tRect);
   
   //
   // paint cells
   //
   drawCells(pa_tGraphics, lc_tRect);
   
   //
   // draw helplines, if needed
   //
   drawHelpLines(pa_tGraphics, lc_tRect);
   
   //
   // draw pattern shape, if needed
   //         
   drawPaletteShape(pa_tGraphics, lc_tRect);
   
   //
   // draw pattern editor shape, if needed
   //         
   drawPaletteEditorShape(pa_tGraphics, lc_tRect);
   
   //
   // draw selection shape, if needed
   //
   drawSelectionShape(pa_tGraphics, lc_tRect);
   
   //
   // draw frame if needed
   //
   drawFrame(pa_tGraphics, lc_tRect);
   
   //
   // draw cell selection frame if needed
   //
   drawCellSelection(pa_tGraphics, lc_tRect);
  } 
 }
 
 
 
 /**
  * Returns whether the panels has a valid cell selction
  * 
  * @return  true, if it has a valid cell selection, false otherwise
  */
 public final boolean hasCellSelection() {return m_bHasCellSelection;}
 
 
   
 
 /**
  * Draws the grid. DO NOT PAINT STROKED LINES, it is damn toooo slow!
  * In screenshot mode (true) we paint everything, otherwise (false) we paint optimized.
  * 
  * @param pa_tGraphics2D     Graphics object to draw on
  * @param pa_tRect           Visible rectangle
  * @param pa_bScreenShotMode 
  */
 private void drawGrid(Graphics2D pa_tGraphics2D, Rectangle pa_tRect)
 {
  int lc_iI = 0;
  int lc_iX = 0;
  int lc_iY = 0;
  
  //
  // instead of just going to #column loops and #row loops and checking if we need to suppress drawing,
  // we compute the area to be drawn. this reduces the amount of loops significantly at huge column and row amounts
  //    
  int lc_iStartColumn = pa_tRect.x / m_iCellSize;
  int lc_iStartRow    = pa_tRect.y / m_iCellSize;
  int lc_iEndColumn   = (pa_tRect.x + pa_tRect.width ) / m_iCellSize;
  int lc_iEndRow      = (pa_tRect.y + pa_tRect.height) / m_iCellSize;
  int lc_iMaxX        = 0;
  int lc_iMaxY        = 0;
  
  int lc_iMaxXScreenshot = 0;
  int lc_iMaxYScreenshot = 0;
  
  if (1 == m_iShowGridLines)
  {
   lc_iStartRow    = (true == m_bScreenshotMode ? 0                            : correctLocation(lc_iStartRow   , m_tProperties.getRows()   -1));
   lc_iEndRow      = (true == m_bScreenshotMode ? m_tProperties.getRows()-1    : correctLocation(lc_iEndRow     , m_tProperties.getRows()   -1));
   lc_iStartColumn = (true == m_bScreenshotMode ? 0                            : correctLocation(lc_iStartColumn, m_tProperties.getColumns()-1));
   lc_iEndColumn   = (true == m_bScreenshotMode ? m_tProperties.getColumns()-1 : correctLocation(lc_iEndColumn  , m_tProperties.getColumns()-1));

   lc_iMaxX = pa_tRect.x + ((1 + Math.min(lc_iEndColumn - lc_iStartColumn, m_tProperties.getColumns()-1)) * m_iCellSize);
   lc_iMaxY = pa_tRect.y + ((1 + Math.min(lc_iEndRow    - lc_iStartRow   , m_tProperties.getRows()   -1)) * m_iCellSize);
  
   lc_iMaxXScreenshot = m_tProperties.getColumns() * m_iCellSize;
   lc_iMaxYScreenshot = m_tProperties.getRows()    * m_iCellSize;
   
   pa_tGraphics2D.setColor(m_tGridColor);
   pa_tGraphics2D.setStroke(m_tLineStrokeSolid);
   
   //
   // paint grid and make sure we do not paint stuff which cannot be seen anyway
   //
   for (lc_iI = lc_iStartColumn; lc_iI <= lc_iEndColumn+1; lc_iI++) // lc_iEndColumn+1 = also draw last line on right and bottom (which is the start line of the next cell index)
   {
    lc_iX = lc_iI * m_iCellSize;
     pa_tGraphics2D.drawLine(lc_iX, (true == m_bScreenshotMode ? 0                  : pa_tRect.y), 
                             lc_iX, (true == m_bScreenshotMode ? lc_iMaxYScreenshot : lc_iMaxY  ));
   }
   for (lc_iI = lc_iStartRow; lc_iI <= lc_iEndRow+1; lc_iI++) // lc_iEndRow+ 1 = also draw last line on right and bottom (which is the start line of the next cell index)
   {
    lc_iY = lc_iI *  m_iCellSize; 
    pa_tGraphics2D.drawLine((true == m_bScreenshotMode ? 0                  : pa_tRect.x), lc_iY, 
                            (true == m_bScreenshotMode ? lc_iMaxXScreenshot : lc_iMaxX  ), lc_iY);
   }
  }
 } 
 

 
 /**
  * Paints the cell selection frame and area
  * 
  * @param pa_tGraphics2D    Graphics object to draw on
  * @param pa_tRect          Visible rectangle
  */
 private void drawCellSelection(Graphics2D pa_tGraphics2D, Rectangle pa_tRect)
 {
  int lc_iX1 = 0;
  int lc_iY1 = 0;
  int lc_iX2 = 0;
  int lc_iY2 = 0;
  int lc_iW  = 0;
  int lc_iH = 0;
  
  Composite lc_tComposite = null;
  
  //
  // don't draw this in screeenshot mode
  //
  if (true == m_bScreenshotMode) return;
  
  if (true == m_bPaintSelectionFrame && true == m_bHasCellSelection)
  {
   lc_iX1 = m_iCellSize * correctLocation(Math.min(m_tCellSelectionPosTopLeft.x, m_tCellSelectionPosBottomRight.x), m_tProperties.getColumns());
   lc_iX2 = m_iCellSize * correctLocation(Math.max(m_tCellSelectionPosTopLeft.x, m_tCellSelectionPosBottomRight.x), m_tProperties.getColumns());
   lc_iY1 = m_iCellSize * correctLocation(Math.min(m_tCellSelectionPosTopLeft.y, m_tCellSelectionPosBottomRight.y), m_tProperties.getRows()   );
   lc_iY2 = m_iCellSize * correctLocation(Math.max(m_tCellSelectionPosTopLeft.y, m_tCellSelectionPosBottomRight.y), m_tProperties.getRows()   );
   lc_iW  = lc_iX2 - lc_iX1;
   lc_iH  = lc_iY2 - lc_iY1;   
   
   pa_tGraphics2D.setStroke(m_tLineStrokeSolid);
   pa_tGraphics2D.setColor(m_tCellMatrixSelectionFrameColor);
   pa_tGraphics2D.drawRect(lc_iX1, lc_iY1, lc_iW, lc_iH);
 
   lc_tComposite = pa_tGraphics2D.getComposite();
   pa_tGraphics2D.setComposite(m_tAlphaComposite);
   pa_tGraphics2D.setColor(m_tCellMatrixSelectionAreaColor);
   pa_tGraphics2D.fillRect(lc_iX1+1, lc_iY1+1, lc_iW - 1, lc_iH - 1);
   pa_tGraphics2D.setComposite(lc_tComposite);
  }
 }
 
 
 
/**
  * Draws the grid
  * 
  * @param pa_tGraphics2D    Graphics object to draw on
  * @param pa_tRect          Visible rectangle
  */
 private void drawHelpLines(Graphics2D pa_tGraphics2D, Rectangle pa_tRect)
 {
  int lc_iMaxX = 0;
  int lc_iX25  = 0;
  int lc_iX50  = 0;
  int lc_iX75  = 0;
  int lc_iMaxY = 0;
  int lc_iY25  = 0;
  int lc_iY50  = 0;
  int lc_iY75  = 0;
     
  int lc_iStartColumn = pa_tRect.x / m_iCellSize;
  int lc_iStartRow    = pa_tRect.y / m_iCellSize;
  int lc_iEndColumn   = (pa_tRect.x + pa_tRect.width ) / m_iCellSize;
  int lc_iEndRow      = (pa_tRect.y + pa_tRect.height) / m_iCellSize;
    
  lc_iStartRow    = correctLocation(lc_iStartRow   , m_tProperties.getRows()   -1);
  lc_iEndRow      = correctLocation(lc_iEndRow     , m_tProperties.getRows()   -1);
  lc_iStartColumn = correctLocation(lc_iStartColumn, m_tProperties.getColumns()-1);
  lc_iEndColumn   = correctLocation(lc_iEndColumn  , m_tProperties.getColumns()-1);

  //
  // don't draw this in screeenshot mode
  //
  if (true == m_bScreenshotMode) return;
  
  pa_tGraphics2D.setStroke(m_tLineStrokeDotted);

  //
  // paint grid and make sure we do not paint stuff which cannot be seen
  //
  if (null != m_tProperties)
  {
   lc_iMaxX = pa_tRect.x + ((1 + Math.min(lc_iEndColumn - lc_iStartColumn, m_tProperties.getColumns()-1)) * m_iCellSize);
   lc_iMaxY = pa_tRect.y + ((1 + Math.min(lc_iEndRow    - lc_iStartRow   , m_tProperties.getRows()   -1)) * m_iCellSize);
   
   lc_iX50  = (m_tProperties.getColumns() / 2) * m_iCellSize;
   lc_iX25  = (m_tProperties.getColumns() / 4) * m_iCellSize;
   lc_iX75  = lc_iX50 + lc_iX25;
   lc_iY50  = (m_tProperties.getRows() / 2) * m_iCellSize;
   lc_iY25  = (m_tProperties.getRows() / 4) * m_iCellSize;
   lc_iY75  = lc_iY50 + lc_iY25;
   
   //
   // helpline 50%
   //
   if (m_iShowHelpLines > 0)
   {
    pa_tGraphics2D.setColor(m_tGridHelpLineColor);
    pa_tGraphics2D.drawLine(lc_iX50, 0, lc_iX50 , lc_iMaxY);
    pa_tGraphics2D.drawLine(0, lc_iY50, lc_iMaxX, lc_iY50 );
   }
   
   //
   // helpline 75% and 25%
   //
   if (2 == m_iShowHelpLines)
   {
    pa_tGraphics2D.setColor(m_tGridHelpLineColor);
    pa_tGraphics2D.drawLine(lc_iX25, 0, lc_iX25 , lc_iMaxY);
    pa_tGraphics2D.drawLine(0, lc_iY25, lc_iMaxX, lc_iY25 );

    pa_tGraphics2D.setColor(m_tGridHelpLineColor);
    pa_tGraphics2D.drawLine(lc_iX75, 0, lc_iX75 , lc_iMaxY);
    pa_tGraphics2D.drawLine(0, lc_iY75, lc_iMaxX, lc_iY75 );
   }
  }
 }
 
 
 
 /**
  * Draws the cells
  * 
  * @param pa_tGraphics2D    Graphics object to draw on
  * @param pa_tRect          Visible rectangle
  */
 private void drawCells(Graphics2D pa_tGraphics2D, Rectangle pa_tRect)
 {
  int lc_iI       = 0;
  int lc_iJ       = 0;
  int lc_iX       = 0;
  int lc_iY       = 0;

  Color  lc_tFillColor = null;
  
  //
  // instead of just going to #column loops and #row loops and checking if we need to suppress drawing,
  // we compute the area to be drawn. this reduces the amount of loops significantly at huge column and row amounts
  //  
  int lc_iStartColumn = pa_tRect.x / m_iCellSize;
  int lc_iStartRow    = pa_tRect.y / m_iCellSize;
  int lc_iEndColumn   = (pa_tRect.x + pa_tRect.width ) / m_iCellSize;
  int lc_iEndRow      = (pa_tRect.y + pa_tRect.height) / m_iCellSize;
   
  lc_iStartRow    = (true == m_bScreenshotMode ? 0                            : correctLocation(lc_iStartRow   , m_tProperties.getRows()   -1));
  lc_iEndRow      = (true == m_bScreenshotMode ? m_tProperties.getRows()-1    : correctLocation(lc_iEndRow     , m_tProperties.getRows()   -1));
  lc_iStartColumn = (true == m_bScreenshotMode ? 0                            : correctLocation(lc_iStartColumn, m_tProperties.getColumns()-1));
  lc_iEndColumn   = (true == m_bScreenshotMode ? m_tProperties.getColumns()-1 : correctLocation(lc_iEndColumn  , m_tProperties.getColumns()-1));
  
  if (null != m_tCells && 0 <= lc_iStartRow && 0 <= lc_iStartColumn && 0 <= lc_iEndRow && 0 <= lc_iEndColumn)
  {
   pa_tGraphics2D.setStroke(m_tLineStrokeSolid);
   for (lc_iI = lc_iStartRow; lc_iI <= lc_iEndRow; lc_iI++)
   {
    lc_iY = lc_iI * m_iCellSize;     
    for (lc_iJ = lc_iStartColumn; lc_iJ <= lc_iEndColumn; lc_iJ++)
    {
     if (0 < m_tCells[lc_iI][lc_iJ]) 
     {
      lc_iX = lc_iJ * m_iCellSize;
        
      //
      // cell border
      //
      if (1 == m_iShowCellBorders)
      {
       pa_tGraphics2D.setColor(m_tCellBorderColor);
       pa_tGraphics2D.drawRect(lc_iX, lc_iY, m_iCellSize, m_iCellSize); 
      }
      
      //
      // cell filling
      //
      lc_tFillColor = m_tCurrentCellFillColor;
      if (null != m_tColorManager)
      {
       //lc_tFillColor = m_tColorManager.getColorForSet(m_tColorManager.getActiveSetName(), m_tCells[lc_iI][lc_iJ]); 
       lc_tFillColor = m_tColorManager.getActiveColor(m_tCells[lc_iI][lc_iJ]); 
      }
      pa_tGraphics2D.setColor(lc_tFillColor);
      
      //
      // for small cell sizes, dont draw hilite
      //
      if (0 == m_iShowCellBorders)
      {
       pa_tGraphics2D.fillRect(lc_iX+1, lc_iY+1, m_iCellSize-1, m_iCellSize-1);  
      }
      else
      {
       if (0 == m_iShowCellHiliteLines || m_iCellSize < GridLifeConstants.GOL_CELL_MIN_SIZE_HILIGHT)
       {
        pa_tGraphics2D.fillRect(lc_iX+1, lc_iY+1, m_iCellSize-1, m_iCellSize-1); 
       }
       else
       {
        pa_tGraphics2D.fillRect(lc_iX+1, lc_iY+1, m_iCellSize-2, m_iCellSize-2);
       
        //
        // cell hilite
        //
        pa_tGraphics2D.setColor(m_tCellHiliteColor);
        pa_tGraphics2D.drawLine(lc_iX + m_iCellSize -1, lc_iY + m_iCellSize -1, lc_iX + 1, lc_iY + m_iCellSize -1);
        pa_tGraphics2D.drawLine(lc_iX + m_iCellSize -1, lc_iY + 1, lc_iX + m_iCellSize -1, lc_iY + m_iCellSize -1);
       }
      }
     }
    } 
   }
  }
 }
 
 
 
 /**
  * Draws a shape at the given position
  * 
  * @param pa_tGraphics2D       graphics object to use
  * @param pa_tRect             visible rectangle
  * @param pa_tShape            shape
  * @param pa_tShapeLocation    shape position
  */
 private void drawShape(Graphics2D pa_tGraphics2D, Rectangle pa_tRect, GridLifeShape pa_tShape, Point pa_tShapeLocation, boolean pa_bPaintShape, boolean pa_bIsPaletteShape)
 {
  int[][] lc_tShapePattern = null;

  int lc_iColumns = 0;
  int lc_iRows    = 0;
  int lc_iI       = 0;
  int lc_iJ       = 0;
  int lc_iX       = 0;
  int lc_iY       = 0;
  
  int lc_iShapeOffsetX = 0;
  int lc_iShapeOffsetY = 0;
  int lc_iShapeStartX  = 0;
  int lc_iShapeStartY  = 0;

  Composite lc_tComposite = null;
  
  if (true == pa_bPaintShape && null !=  pa_tShapeLocation && null != pa_tShape && null != m_tProperties)
  {
   if (null != (lc_tShapePattern = pa_tShape.getShapeDataIntArray()))  
   {
    lc_iColumns      = pa_tShape.getShapeColumns() ;
    lc_iRows         = pa_tShape.getShapeRows()    ;
    
    lc_iShapeOffsetX = pa_tShapeLocation.x * m_iCellSize;
    lc_iShapeOffsetY = pa_tShapeLocation.y * m_iCellSize;

    pa_tGraphics2D.setPaintMode();
    pa_tGraphics2D.setStroke(m_tLineStrokeSolid);
    
    //
    // draw frame background, must be overlayed by cells to see their color
    //
    lc_tComposite = pa_tGraphics2D.getComposite();
    pa_tGraphics2D.setComposite(m_tAlphaComposite);
    pa_tGraphics2D.setColor(m_tCellMatrixInsertionAreaColor);
    pa_tGraphics2D.fillRect(lc_iShapeOffsetX, lc_iShapeOffsetY, lc_iColumns * m_iCellSize, lc_iRows * m_iCellSize);
    pa_tGraphics2D.setComposite(lc_tComposite);
    
    //
    // draw cells
    //
    for (lc_iI = 0; lc_iI < lc_iRows; lc_iI++)
    {
     lc_iY           = lc_iI * m_iCellSize;
     lc_iShapeStartY = lc_iShapeOffsetY + lc_iI;

     if (0 <= pa_tShapeLocation.y + lc_iI && m_tProperties.getRows() > pa_tShapeLocation.y + lc_iI)
     {
      if (true == isCellVisibleVertically(lc_iShapeStartY, pa_tRect))
      {
       for (lc_iJ = 0; lc_iJ < lc_iColumns; lc_iJ++)
       {
        lc_iX           = lc_iJ * m_iCellSize;
        lc_iShapeStartX = lc_iShapeOffsetX + lc_iX; 

        if (0 <= pa_tShapeLocation.x + lc_iJ && m_tProperties.getColumns() > pa_tShapeLocation.x + lc_iJ)
        {            
         if (true == isCellVisibleHorizontally(lc_iShapeStartX, pa_tRect))
         {
          if (0 != lc_tShapePattern[lc_iI][lc_iJ])
          {
           pa_tGraphics2D.setColor(m_tCellBorderColor);
           pa_tGraphics2D.drawRect(lc_iShapeOffsetX + lc_iX, lc_iShapeOffsetY + lc_iY, m_iCellSize, m_iCellSize);    
           
           //
           // if we paint shapes in XOR mode, any living cell covered by a new cell
           // has to be erased, in this case we set the fill color to the bg color
           //
           pa_tGraphics2D.setColor(m_tCurrentCellFillColor);
           if (false == pa_bIsPaletteShape)
           {
            if (null != m_tColorManager)
            {
             pa_tGraphics2D.setColor(m_tColorManager.getActiveColor(lc_tShapePattern[lc_iI][lc_iJ])); 
            }
           }
           
           if (true == m_bPaintShapeModeXor)
           {
            if (0 < m_tCells[pa_tShapeLocation.y + lc_iI][pa_tShapeLocation.x + lc_iJ])
            {
             //
             // redraw single cell frame dotted, then set fill color to background and stroke to solid
             //
             pa_tGraphics2D.setStroke(m_tLineStrokeDotted);
             pa_tGraphics2D.setColor(m_tGridColor);
             pa_tGraphics2D.drawRect(lc_iShapeOffsetX + lc_iX, lc_iShapeOffsetY + lc_iY, m_iCellSize, m_iCellSize);             
             pa_tGraphics2D.setStroke(m_tLineStrokeSolid);
             pa_tGraphics2D.setColor(m_tColorBackground); 
            }
           }
           pa_tGraphics2D.fillRect(lc_iShapeOffsetX + lc_iX + 1, lc_iShapeOffsetY + lc_iY + 1, m_iCellSize - 1, m_iCellSize - 1);         
          }
         }
        }
       } // for lc_iJ
      }
     }
    } // for lc_iI  
    
    //
    // draw surrounding frame
    //
    pa_tGraphics2D.setColor(m_tCellMatrixInsertionFrameColor);
    if (true == m_bPaintShapeModeXor)
    {
     pa_tGraphics2D.setStroke(m_tLineStrokeDotted); 
    }
    pa_tGraphics2D.drawRect(lc_iShapeOffsetX, lc_iShapeOffsetY, lc_iColumns * m_iCellSize, lc_iRows * m_iCellSize);       
    pa_tGraphics2D.setStroke(m_tLineStrokeSolid);
   } 
  }   
 }
 
 
 
 /**
  * Draws the palette shape
  * 
  * @param pa_tGraphics2D    Graphics object to draw on
  * @param pa_tRect          Visible rectangle
  */
 private void drawSelectionShape(Graphics2D pa_tGraphics2D, Rectangle pa_tRect)
 {
  //
  // don't draw this in screeenshot mode
  //
  if (false == m_bScreenshotMode) 
    drawShape(pa_tGraphics2D, pa_tRect, m_tSelectionShape, m_tSelectionShapePos, m_bPaintSelectionShape, false);
 }
 
 
 
 /**
  * Draws the palette shape
  * 
  * @param pa_tGraphics2D    Graphics object to draw on
  * @param pa_tRect          Visible rectangle
  */
 private void drawPaletteEditorShape(Graphics2D pa_tGraphics2D, Rectangle pa_tRect)
 {
  //
  // don't draw this in screeenshot mode
  //
  if (false == m_bScreenshotMode) 
    drawShape(pa_tGraphics2D, pa_tRect, m_tPaletteEditorShape, m_tPaletteEditorShapePos, m_bPaintPaletteEditorShape, true);
 }

 
 
 /**
  * Draws the palette shape
  * 
  * @param pa_tGraphics2D    Graphics object to draw on
  * @param pa_tRect          Visible rectangle
  */
 private void drawPaletteShape(Graphics2D pa_tGraphics2D, Rectangle pa_tRect)
 {
  //
  // don't draw this in screeenshot mode
  //
  if (false == m_bScreenshotMode)
    drawShape(pa_tGraphics2D, pa_tRect, m_tPaletteShape, m_tPaletteShapePos, m_bPaintPaletteShape, true);
 }
 
 
 
 /**
  * Draws the position frame
  * 
  * @param pa_tGraphics2D    Graphics object to draw on
  * @param pa_tRect          Visible rectangle
  */
 private void drawFrame(Graphics2D pa_tGraphics2D, Rectangle pa_tRect)
 {
  int lc_iX = 0;
  int lc_iY = 0;
  
  //
  // don't draw this in screeenshot mode
  //
  if (true == m_bScreenshotMode) return;
  
  if (true == m_bPaintFrame && null !=  m_tFramePos)
  {
   lc_iX = m_tFramePos.x * m_iCellSize;
   lc_iY = m_tFramePos.y * m_iCellSize;
   
   if (m_tFramePos.x < m_tProperties.getColumns())
   {
    if (m_tFramePos.y < m_tProperties.getRows())
    {
     if (true == isCellVisibleVertically(lc_iY, pa_tRect))
     {
      if (true == isCellVisibleHorizontally(lc_iX, pa_tRect))
      {
       pa_tGraphics2D.setColor(m_tCellMatrixInsertionAreaColor);
       pa_tGraphics2D.setStroke(m_tLineStrokeSolid); 
       if (true == m_bPaintShapeModeXor)
       {
        pa_tGraphics2D.setStroke(m_tLineStrokeDotted);     
       }
       pa_tGraphics2D.drawRect(lc_iX, lc_iY, m_iCellSize, m_iCellSize); 
       
       pa_tGraphics2D.setColor(m_tCurrentCellFillColor); 
       pa_tGraphics2D.setStroke(m_tLineStrokeSolid); 
        
       if (true == m_bPaintShapeModeXor)
       {
        if (0 < m_tCells[m_tFramePos.y][m_tFramePos.x])
        {     
         pa_tGraphics2D.setColor(m_tColorBackground);  
        }
       }
       pa_tGraphics2D.fillRect(lc_iX + 1, lc_iY + 1, m_iCellSize - 1, m_iCellSize - 1);
      }
     }
    }
   }
  } 
 }
 
 
 
 /**
  * Sets the given shape as palette shape to paint
  * 
  * @param pa_tShape   Shape to paint when mouse is inside grid
  */
 public final void setPaletteShape(GridLifeShape pa_tShape)
 {
  if (null != pa_tShape)
  {
   m_tPaletteShape      = new GridLifeShape(pa_tShape, m_tColorManager, false); // don't use the reference, create a new shape, otherwise rotating will cause interesting effects in shaoe list...
   m_bHasPaletteShape   = (null != m_tPaletteShape);
  }
  else
  {
   m_tPaletteShape = null; 
   m_bHasPaletteShape = false;
   m_bPaintPaletteShape = false;
  }
 }
 
 
 
 
 /**
  * Sets the given shape as palette shape to paint
  * 
  * @param pa_tShape   Shape to paint when mouse is inside grid
  */
 public final void setPaletteEditorShape(GridLifeShape pa_tShape)
 {
  if (null != pa_tShape)
  {
   m_tPaletteEditorShape    = new GridLifeShape(pa_tShape, m_tColorManager, false); // don't use the reference, create a new shape, otherwise rotating will cause interesting effects in shaoe list...
   m_bHasPaletteEditorShape = (null != m_tPaletteEditorShape);
  }
  else
  {
   m_tPaletteEditorShape = null; 
   m_bHasPaletteEditorShape = false;
   m_bPaintPaletteEditorShape = false;
  }
 }
 
 
 
 /**
  * Sets the given shape as selection shape to paint
  * 
  * @param pa_tShape   Shape to paint when mouse is inside grid
  */
 public final void setSelectionShape(GridLifeShape pa_tShape)
 {
  if (null != pa_tShape)
  {
   m_tSelectionShape      = new GridLifeShape(pa_tShape, m_tColorManager, false); // don't use the reference, create a new shape, otherwise rotating will cause interesting effects in shaoe list...
   m_bHasSelectionShape   = (null != m_tSelectionShape);
  }
  else
  {
   m_tSelectionShape = null; 
   m_bHasSelectionShape = false;
   m_bPaintSelectionShape = false;
  }
 }
 
 
 
 /**
  * Sets whether a palette shape should be painted or not
  * 
  * @param pa_bStatus    true, if the palette shape should be painted, false otherwise
  */
 public final void setPaintPaletteShape(boolean pa_bStatus)
 {
  m_bPaintPaletteShape = pa_bStatus; 
 }

 

 /**
  * Sets whether a palette editor shape should be painted or not
  * 
  * @param pa_bStatus    true, if the palette editor shape should be painted, false otherwise
  */
 public final void setPaintPaletteEditorShape(boolean pa_bStatus)
 {
  m_bPaintPaletteEditorShape = pa_bStatus; 
 }

 
 
 /**
  * Sets whether a selection should be painted or not
  * 
  * @param pa_bStatus    true, if the selection shape should be painted, false otherwise
  */
 public final void setPaintSelectionShape(boolean pa_bStatus)
 {
  m_bPaintSelectionShape = pa_bStatus; 
 }


 
 /**
  * Checks whether the shape should be painted or erased. If any change to painting needs to be made in
  * order to reflect the shape state properly, true is returned. A shape is marked for drawing only if
  * not a single part of it lies outside the visible area.
  * 
  * @param pa_tGridPosition   Grid coordinates of mouse
  * @param pa_bMouseStatus    true, if it was a MouseEnteredEvent, false if it was a MouseExited event
  * @return   true, if a repaint is needed, false otherwise
  */
 public final boolean checkSelectionShapePosition(Point pa_tGridPosition, boolean pa_bMouseStatus)
 {
  boolean lc_bIsVisible   = false;
  boolean lc_bNeedsRedraw = false;
  
  int lc_iShapeColumns = 0;
  int lc_iShapeRows    = 0;
  int lc_iI            = 0;
  int lc_iJ            = 0;
  int lc_iX            = 0;
  int lc_iY            = 0;
  int lc_iShapeStartX  = 0;
  int lc_iShapeStartY  = 0;
  int lc_iShapeOffsetX = 0;
  int lc_iShapeOffsetY = 0;
  int lc_iCount        = 0;
  
  Rectangle lc_tRect = null;
  
  if (null != pa_tGridPosition && null != m_tSelectionShape && null != m_tProperties)
  {
   lc_iShapeColumns = m_tSelectionShape.getShapeColumns();
   lc_iShapeRows    = m_tSelectionShape.getShapeRows()   ;
   
   if (null != m_tScrollPane)
   {
    if (null != m_tScrollPane.getViewport())
    {
     lc_tRect         = m_tScrollPane.getViewport().getViewRect();
     lc_iShapeOffsetX = pa_tGridPosition.x * m_iCellSize;
     lc_iShapeOffsetY = pa_tGridPosition.y * m_iCellSize;
     
     // 
     // check whether any of the shape cells might be visible
     //
     for (lc_iI = 0; lc_iI < lc_iShapeRows; lc_iI++)
     {    
      lc_iY           = lc_iI * m_iCellSize;
      lc_iShapeStartY = lc_iShapeOffsetY + lc_iY;

      if (0 <= pa_tGridPosition.y + lc_iI && m_tProperties.getRows() > pa_tGridPosition.y + lc_iI)
      {
       for (lc_iJ = 0; lc_iJ < lc_iShapeColumns; lc_iJ++)
       {
        lc_iX           = lc_iJ * m_iCellSize;
        lc_iShapeStartX = lc_iShapeOffsetX + lc_iX;

        if (0 <= pa_tGridPosition.x + lc_iJ && m_tProperties.getColumns() > pa_tGridPosition.x + lc_iJ)
        {
         if (0 != m_tSelectionShape.getShapePattern(lc_iJ, lc_iI))
         { 
          lc_iCount++; 
          lc_bIsVisible |= isCellVisibleHorizontally(lc_iShapeStartX, lc_tRect);
          lc_bIsVisible |= isCellVisibleVertically  (lc_iShapeStartY, lc_tRect);
         }
        }
       }
      }
     } 
     //
     // special case: leaving zone must erase shape
     //
     if (true == m_bPaintSelectionShape && false == pa_bMouseStatus)
     {
      m_bPaintSelectionShape = false; 
      lc_bNeedsRedraw = true;       
     }
     else
     {
      //
      // did we paint before?
      //
      if (true == m_bPaintSelectionShape)
      {  
       if (true == lc_bIsVisible)
       {
        if (!(m_tSelectionShapePos.x == pa_tGridPosition.x && m_tSelectionShapePos.y == pa_tGridPosition.y))
        {
         m_tSelectionShapePos = pa_tGridPosition;
         lc_bNeedsRedraw = true;
        }
       }
       else
       {
        m_bPaintSelectionShape = false;
        lc_bNeedsRedraw = true;
       }
      }
      else
      {
       if (true == lc_bIsVisible)
       {        
        m_tSelectionShapePos     = pa_tGridPosition; 
        m_bPaintSelectionShape   = true; 
        lc_bNeedsRedraw = true;
       }
      }
     } 
    }    
   }
  }
  return lc_bNeedsRedraw;
 }
 

 
 /**
  * Checks whether the shape should be painted or erased. If any change to painting needs to be made in
  * order to reflect the shape state properly, true is returned. A shape is marked for drawing only if
  * not a single part of it lies outside the visible area.
  * 
  * @param pa_tGridPosition   Grid coordinates of mouse
  * @param pa_bMouseStatus    true, if it was a MouseEnteredEvent, false if it was a MouseExited event
  * @return   true, if a repaint is needed, false otherwise
  */
 public final boolean checkPaletteShapePosition(Point pa_tGridPosition, boolean pa_bMouseStatus)
 {
  boolean lc_bIsVisible   = false;
  boolean lc_bNeedsRedraw = false;
  
  int lc_iShapeColumns = 0;
  int lc_iShapeRows    = 0;
  int lc_iI            = 0;
  int lc_iJ            = 0;
  int lc_iX            = 0;
  int lc_iY            = 0;
  int lc_iShapeStartX  = 0;
  int lc_iShapeStartY  = 0;
  int lc_iShapeOffsetX = 0;
  int lc_iShapeOffsetY = 0;
  int lc_iCount        = 0;
  
  Rectangle lc_tRect = null;
  
  if (null != pa_tGridPosition && null != m_tPaletteShape && null != m_tProperties)
  {
   lc_iShapeColumns = m_tPaletteShape.getShapeColumns();
   lc_iShapeRows    = m_tPaletteShape.getShapeRows()   ;
   
   if (null != m_tScrollPane)
   {
    if (null != m_tScrollPane.getViewport())
    {
     lc_tRect         = m_tScrollPane.getViewport().getViewRect();
     lc_iShapeOffsetX = pa_tGridPosition.x * m_iCellSize;
     lc_iShapeOffsetY = pa_tGridPosition.y * m_iCellSize;
     
     // 
     // check whether any of the shape cells might be visible
     //
     for (lc_iI = 0; lc_iI < lc_iShapeRows; lc_iI++)
     {    
      lc_iY           = lc_iI * m_iCellSize;
      lc_iShapeStartY = lc_iShapeOffsetY + lc_iY;

      if (0 <= pa_tGridPosition.y + lc_iI && m_tProperties.getRows() > pa_tGridPosition.y + lc_iI)
      {
       for (lc_iJ = 0; lc_iJ < lc_iShapeColumns; lc_iJ++)
       {
        lc_iX           = lc_iJ * m_iCellSize;
        lc_iShapeStartX = lc_iShapeOffsetX + lc_iX;

        if (0 <= pa_tGridPosition.x + lc_iJ && m_tProperties.getColumns() > pa_tGridPosition.x + lc_iJ)
        {
         if (0 != m_tPaletteShape.getShapePattern(lc_iJ, lc_iI))
         { 
          lc_iCount++; 
          lc_bIsVisible |= isCellVisibleHorizontally(lc_iShapeStartX, lc_tRect);
          lc_bIsVisible |= isCellVisibleVertically  (lc_iShapeStartY, lc_tRect);
         }
        }
       }
      }
     } 
     //
     // special case: leaving zone must erase shape
     //
     if (true == m_bPaintPaletteShape && false == pa_bMouseStatus)
     {
      m_bPaintPaletteShape = false; 
      lc_bNeedsRedraw = true;       
     }
     else
     {
      //
      // did we paint before?
      //
      if (true == m_bPaintPaletteShape)
      {  
       if (true == lc_bIsVisible)
       {
        if (!(m_tPaletteShapePos.x == pa_tGridPosition.x && m_tPaletteShapePos.y == pa_tGridPosition.y))
        {
         m_tPaletteShapePos = pa_tGridPosition;
         lc_bNeedsRedraw = true;
        }
       }
       else
       {
        m_bPaintPaletteShape = false;
        lc_bNeedsRedraw = true;
       }
      }
      else
      {
       if (true == lc_bIsVisible)
       {        
        m_tPaletteShapePos     = pa_tGridPosition; 
        m_bPaintPaletteShape   = true; 
        lc_bNeedsRedraw = true;
       }
      }
     } 
    }    
   }
  }
  return lc_bNeedsRedraw;
 }
 
 
 
 
 /**
  * Checks whether the shape should be painted or erased. If any change to painting needs to be made in
  * order to reflect the shape state properly, true is returned. A shape is marked for drawing only if
  * not a single part of it lies outside the visible area.
  * 
  * @param pa_tGridPosition   Grid coordinates of mouse
  * @param pa_bMouseStatus    true, if it was a MouseEnteredEvent, false if it was a MouseExited event
  * @return   true, if a repaint is needed, false otherwise
  */
 public final boolean checkPaletteEditorShapePosition(Point pa_tGridPosition, boolean pa_bMouseStatus)
 {
  boolean lc_bIsVisible   = false;
  boolean lc_bNeedsRedraw = false;
  
  int lc_iShapeColumns = 0;
  int lc_iShapeRows    = 0;
  int lc_iI            = 0;
  int lc_iJ            = 0;
  int lc_iX            = 0;
  int lc_iY            = 0;
  int lc_iShapeStartX  = 0;
  int lc_iShapeStartY  = 0;
  int lc_iShapeOffsetX = 0;
  int lc_iShapeOffsetY = 0;
  int lc_iCount        = 0;
  
  Rectangle lc_tRect = null;
  
  if (null != pa_tGridPosition && null != m_tPaletteEditorShape && null != m_tProperties)
  {
   lc_iShapeColumns = m_tPaletteEditorShape.getShapeColumns();
   lc_iShapeRows    = m_tPaletteEditorShape.getShapeRows()   ;
   
   if (null != m_tScrollPane)
   {
    if (null != m_tScrollPane.getViewport())
    {
     lc_tRect         = m_tScrollPane.getViewport().getViewRect();
     lc_iShapeOffsetX = pa_tGridPosition.x * m_iCellSize;
     lc_iShapeOffsetY = pa_tGridPosition.y * m_iCellSize;
     
     // 
     // check whether any of the shape cells might be visible
     //
     for (lc_iI = 0; lc_iI < lc_iShapeRows; lc_iI++)
     {    
      lc_iY           = lc_iI * m_iCellSize;
      lc_iShapeStartY = lc_iShapeOffsetY + lc_iY;

      if (0 <= pa_tGridPosition.y + lc_iI && m_tProperties.getRows() > pa_tGridPosition.y + lc_iI)
      {
       for (lc_iJ = 0; lc_iJ < lc_iShapeColumns; lc_iJ++)
       {
        lc_iX           = lc_iJ * m_iCellSize;
        lc_iShapeStartX = lc_iShapeOffsetX + lc_iX;

        if (0 <= pa_tGridPosition.x + lc_iJ && m_tProperties.getColumns() > pa_tGridPosition.x + lc_iJ)
        {
         if (0 != m_tPaletteEditorShape.getShapePattern(lc_iJ, lc_iI))
         { 
          lc_iCount++; 
          lc_bIsVisible |= isCellVisibleHorizontally(lc_iShapeStartX, lc_tRect);
          lc_bIsVisible |= isCellVisibleVertically  (lc_iShapeStartY, lc_tRect);
         }
        }
       }
      }
     } 
     //
     // special case: leaving zone must erase shape
     //
     if (true == m_bPaintPaletteEditorShape && false == pa_bMouseStatus)
     {
      m_bPaintPaletteEditorShape = false; 
      lc_bNeedsRedraw = true;       
     }
     else
     {
      //
      // did we paint before?
      //
      if (true == m_bPaintPaletteEditorShape)
      {  
       if (true == lc_bIsVisible)
       {
        if (!(m_tPaletteEditorShapePos.x == pa_tGridPosition.x && m_tPaletteEditorShapePos.y == pa_tGridPosition.y))
        {
         m_tPaletteEditorShapePos = pa_tGridPosition;
         lc_bNeedsRedraw = true;
        }
       }
       else
       {
        m_bPaintPaletteEditorShape = false;
        lc_bNeedsRedraw = true;
       }
      }
      else
      {
       if (true == lc_bIsVisible)
       {        
        m_tPaletteEditorShapePos     = pa_tGridPosition; 
        m_bPaintPaletteEditorShape   = true; 
        lc_bNeedsRedraw = true;
       }
      }
     } 
    }    
   }
  }
  return lc_bNeedsRedraw;
 }

 
 
 /**
  * Returns whether a palette shape is set
  * 
  * @return  true if a palette shape is set, false otherwise
  */
 public final boolean hasPaletteShape() {return m_bHasPaletteShape;}

 
 
 
 /**
  * Returns whether a palette shape is set
  * 
  * @return  true if a palette shape is set, false otherwise
  */
 public final boolean hasPaletteEditorShape() {return m_bHasPaletteEditorShape;}
 
 
 
 /**
  * Returns whether a selection shape is set
  * 
  * @return  true if a selection shape is set, false otherwise
  */
 public boolean hasSelectionShape() {return m_bHasSelectionShape;}
 
 
 
 /**
  * Returns the palette shape grid position
  * 
  * @return   shape grid position
  */
 public final Point getPaletteShapePosition() {return m_tPaletteShapePos;}
 
 
 
 /**
  * Returns the palette shape grid position
  * 
  * @return   shape grid position
  */
 public final Point getPaletteEditorShapePosition() {return m_tPaletteEditorShapePos;}
 
 

 
 /**
  * Returns the palette shape grid position
  * 
  * @return   shape grid position
  */
 public final Point getSelectionShapePosition() {return m_tSelectionShapePos;}
 
 
 
 /**
  * Determines whether a frame is painted or not
  * 
  * @param pa_bMode 
  */
 public final void setPaintFrame(boolean pa_bMode)
 {
  m_bPaintFrame = pa_bMode;  
 }
 
 
 
 /**
  * Determines whether a cell selection frame and area are painted or not
  * 
  * @param pa_bMode 
  */
 public final void setPaintCellSelectionFrame(boolean pa_bMode)
 {
  m_bPaintSelectionFrame = pa_bMode;  
 }
 
 
 /**
  * Returns the position of the bottom right selection point
  * 
  * @return   position
  */
 public final Point getCellSelectionEndPosition() {return m_tCellSelectionPosBottomRight;}
 
 
 
/**
 * Sets the matrix cell seleciton position to the new values
 * 
 * @param pa_tPosition   new position 
 */
 public final void setCellSelectionEndPosition(Point pa_tPosition)
 {
  //
  // make sure we stay inside matrix!
  //
  m_tCellSelectionPosBottomRight.x = correctLocation(pa_tPosition.x, m_tProperties.getColumns()); 
  m_tCellSelectionPosBottomRight.y = correctLocation(pa_tPosition.y, m_tProperties.getRows()   );
  m_bHasCellSelection = false;
  
  //
  // a selection must cover at least one single cell
  //
  if (Math.abs(m_tCellSelectionPosTopLeft.x - m_tCellSelectionPosBottomRight.x) >= 1)
  {
   if (Math.abs(m_tCellSelectionPosTopLeft.y - m_tCellSelectionPosBottomRight.y) >= 1)
   {
    m_bHasCellSelection = true; 
    m_bPaintSelectionFrame = true;
   }
  }
 }
 

 
/**
 * Returns the position of the bottom right selection point
 * 
 * @return   position
 */
 public final Point getCellSelectionStartPosition() {return m_tCellSelectionPosTopLeft;}

 
 
 /**
 * Sets the matrix cell seleciton position to the new values
 * 
 * @param pa_tPosition   new position 
 */
 public final void setCellSelectionStartPosition(Point pa_tPosition)
 {
  m_tCellSelectionPosTopLeft.x = pa_tPosition.x;
  m_tCellSelectionPosTopLeft.y = pa_tPosition.y;
 }
 

 /**
  * Clears the cell selection
  * 
  */
 public final void clearCellSelection()
 {
  m_tCellSelectionPosBottomRight.x = 1;
  m_tCellSelectionPosBottomRight.y = 1;
  m_tCellSelectionPosTopLeft.x     = 1;
  m_tCellSelectionPosTopLeft.y     = 1;

  m_bHasCellSelection = false; 
  m_bPaintSelectionFrame = false;
 }
 
 
  /**
  * Clears the cell selection shape
  * 
  */
 public final void clearSelectionShape()
 {
  m_tSelectionShape      = null;
  m_bHasSelectionShape   = false;
  m_bPaintSelectionShape = false;
 }
 
 
  /**
  * Clears the palette editor shape
  * 
  */
 public final void clearPaletteEditorShape()
 {
  m_tPaletteEditorShape      = null;
  m_bHasPaletteEditorShape   = false;
  m_bPaintPaletteEditorShape = false;
 } 
 
 /**
  * Sets the frame cell position to the new values
  * 
  * @param pa_tPosition   new position 
  */
 public final void setFramePosition(Point pa_tPosition)
 {
  m_tFramePos.x = pa_tPosition.x; 
  m_tFramePos.y = pa_tPosition.y; 
 }

 
 
 /**
  * Sets the palette shape position to the new values
  * 
  * @param pa_tPosition   new position 
  */
 public final void setPaletteShapePosition(Point pa_tPosition)
 {
  m_tPaletteShapePos.x = pa_tPosition.x; 
  m_tPaletteShapePos.y = pa_tPosition.y; 
 }

 
 
 /**
  * Sets the palette shape position to the new values
  * 
  * @param pa_tPosition   new position 
  */
 public final void setPaletteEditorShapePosition(Point pa_tPosition)
 {
  m_tPaletteEditorShapePos.x = pa_tPosition.x; 
  m_tPaletteEditorShapePos.y = pa_tPosition.y; 
 }

 
 
 /**
  * Sets the selection shape position to the new values
  * 
  * @param pa_tPosition   new position 
  */
 public final void setSelectionShapePosition(Point pa_tPosition)
 {
  m_tSelectionShapePos.x = pa_tPosition.x; 
  m_tSelectionShapePos.y = pa_tPosition.y; 
 }
 
 
 
 /**
  * Checks whether the frame should be painted or erased. If any change to painting needs to be made in
  * order to reflect the frame state properly, true is returned.
  * 
  * @param pa_tGridPosition   Grid coordinates of mouse
  * @param pa_bMouseStatus    true, if it was a MouseEnteredEvent, false if it was a MouseExited event
  * @return   true, if a repaint is needed, false otherwise
  */
 public final boolean checkFramePosition(Point pa_tGridPosition, boolean pa_bMouseStatus)
 {
  boolean lc_bNeedsRedraw = false;
  boolean lc_bIsVisible   = false;
  
  int lc_iX            = 0;
  int lc_iY            = 0;
  
  Rectangle lc_tRect = null;
  
  if (null != pa_tGridPosition)
  {
   if (null != m_tScrollPane)
   {
    if (null != m_tScrollPane.getViewport())
    {
     lc_tRect = m_tScrollPane.getViewport().getViewRect();
     lc_iX    = pa_tGridPosition.x * m_iCellSize;
     lc_iY    = pa_tGridPosition.y * m_iCellSize;
     
     // 
     // check whether the frame might be visible
     //
     if (true == isCellVisibleHorizontally(lc_iX, lc_tRect))
     {
      if (true == isCellVisibleVertically(lc_iY, lc_tRect))
      {
       lc_bIsVisible   = true;
      }
     }
    }
    
    if (true == m_bPaintFrame && false == pa_bMouseStatus)
    {
     m_bPaintFrame   = false;
     lc_bNeedsRedraw = true;
    }
    else
    {      
     //
     // did we paint before?
     //
     if (true == m_bPaintFrame)
     {  
      if (true == lc_bIsVisible)
      {
       if (!(m_tFramePos.x == pa_tGridPosition.x && m_tFramePos.y == pa_tGridPosition.y))
       {
        m_tFramePos = pa_tGridPosition;
        lc_bNeedsRedraw = true;
       }
      }
      else
      {
       m_bPaintFrame = false;
       lc_bNeedsRedraw = true;
      }
     }
     else
     {
      if (true == lc_bIsVisible)
      {        
       m_tFramePos     = pa_tGridPosition; 
       m_bPaintFrame   = true; 
       lc_bNeedsRedraw = true;
      }
     }
    }
   }
  }
  return lc_bNeedsRedraw;
 }
 

 
 /**
  * Sets the properties
  * 
  * @param pa_tProperties 
  */
 @Override
 public void setProperties(GridLifeProperties pa_tProperties)
 {
  super.setProperties(pa_tProperties);
  
  setShowHelpLines      (pa_tProperties.getShowHelpLines()      );
  setShowGridLines      (pa_tProperties.getShowGridLines()      );
  setShowCellHiliteLines(pa_tProperties.getShowCellHiliteLines());
  setShowCellBorders    (pa_tProperties.getShowCellBorders()    );
             
  setColorBg                (pa_tProperties.getColorBg()                );
  setColorCellHilightLines  (pa_tProperties.getColorCellHilightLines()  );
  setColorGridLines         (pa_tProperties.getColorGridLines()         );
  setColorHelpLines         (pa_tProperties.getColorHelpLines()         );
  setColorCellBorders       (pa_tProperties.getColorCellBorders()       );
  setColorCellSelectionFrame(pa_tProperties.getColorCellSelectionFrame());
  setColorCellSelectionArea (pa_tProperties.getColorCellSelectionArea() );
  setColorCellInsertionFrame(pa_tProperties.getColorCellInsertionFrame());
  setColorCellInsertionArea (pa_tProperties.getColorCellInsertionArea() );
 }
 
 
 
 /**
  * Scrolls to the given position
  * 
  * @param pa_tPosition    position to scroll to
  */
 public final void scrollTo(Point pa_tPosition)
 {
  int lc_iX = 0;
  int lc_iY = 0;
  
  if (null != pa_tPosition)
  {
   if (null != m_tScrollPane)
   {
    if (null != m_tScrollPane.getViewport())
    {
     lc_iX = pa_tPosition.x * m_iCellSize;
     lc_iY = pa_tPosition.y * m_iCellSize;
     
     scrollRectToVisible(new Rectangle(lc_iX, lc_iY, 1, 1));
    }
   }
  }
 }
 
 
 
 /**
  * Returns the x-coordinate of the top left visible cell
  * 
  * @return x-coordinate, 0..
  */
 public final int getTopLeftX()
 {
  Rectangle lc_tRect = null;
  
  if (null != m_tScrollPane)
  {
   if (null != m_tScrollPane.getViewport())
   {
    if (null != (lc_tRect = m_tScrollPane.getViewport().getViewRect())) 
    {
     return lc_tRect.x; 
    }
   }
  }
  return 0;
 }
 

 
 /**
  * Returns the x-coordinate of the top left visible cell
  * 
  * @return x-coordinate, 0..
  */
 public final int getTopLeftY()
 {
  Rectangle lc_tRect = null;
  
  if (null != m_tScrollPane)
  {
   if (null != m_tScrollPane.getViewport())
   {
    if (null != (lc_tRect = m_tScrollPane.getViewport().getViewRect())) 
    {
     return lc_tRect.y; 
    }
   }
  }
  return 0;
 }

 
 
 /**
  * Checks whether the point is visible. If not, it scroll to it.
  * 
  * @param pa_tPoint          Point 
  * @param pa_bForceScrolling if set to true, scrolling will be enforced.
  */
 public final void checkScrollToPosition(Point pa_tPoint, boolean pa_bForceScrolling)
 {
  Rectangle lc_tRect     = null;
  Point     lc_tNewPoint = new Point(0,0);
  
  if (null != pa_tPoint)
  {
   lc_tNewPoint.x = (0 > pa_tPoint.x ? 0 : pa_tPoint.x);
   lc_tNewPoint.y = (0 > pa_tPoint.y ? 0 : pa_tPoint.y);
   
   if (null != m_tScrollPane)
   {
    if (null != m_tScrollPane.getViewport())
    {
     if (null != (lc_tRect = m_tScrollPane.getViewport().getViewRect()))
     {
      if (true == pa_bForceScrolling || false == isFramePointVisible(lc_tNewPoint.x, lc_tNewPoint.y, lc_tRect))
      {
       m_tScrollPane.getViewport().setViewPosition(lc_tNewPoint);
      }
     }
    }
   }
  }
 }
 
 
 /**
  * Sets the view as if it was scrolled in to see the currently active cell selection. Returns 
  * the computed zoom factor needed
  * 
  */
 public final int getZoomFactorForSelectionZoom(int pa_iMaxZoom)
 {
  Rectangle lc_tRect = null;
  
  int lc_iX1   = 0;
  int lc_iY1   = 0;
  int lc_iX2   = 0;
  int lc_iY2   = 0;
  int lc_iW    = 0;
  int lc_iH    = 0;
  int lc_iZoom = 0;
  
  //
  // if we have a selection, compute its cell coordinates
  //
  if (true == m_bPaintSelectionFrame && true == m_bHasCellSelection)
  {
   lc_iX1 = correctLocation(Math.min(m_tCellSelectionPosTopLeft.x, m_tCellSelectionPosBottomRight.x), m_tProperties.getColumns());
   lc_iX2 = correctLocation(Math.max(m_tCellSelectionPosTopLeft.x, m_tCellSelectionPosBottomRight.x), m_tProperties.getColumns());
   lc_iY1 = correctLocation(Math.min(m_tCellSelectionPosTopLeft.y, m_tCellSelectionPosBottomRight.y), m_tProperties.getRows()   );
   lc_iY2 = correctLocation(Math.max(m_tCellSelectionPosTopLeft.y, m_tCellSelectionPosBottomRight.y), m_tProperties.getRows()   );
   lc_iW  = lc_iX2 - lc_iX1;  if (0 > lc_iW) lc_iW = GridLifeConstants.GOL_MIN_COLUMNS;
   lc_iH  = lc_iY2 - lc_iY1;  if (0 > lc_iH) lc_iH = GridLifeConstants.GOL_MIN_ROWS   ;   
  
   //
   // get visible rect for later comparisions
   //
   if (null != m_tScrollPane)
   {
    if (null != m_tScrollPane.getViewport())
    {
     if (null != (lc_tRect = m_tScrollPane.getViewport().getViewRect()))
     {
      lc_iZoom = (Math.min( (int) ((lc_tRect.width + 0.5f) / lc_iW), (int) ((lc_tRect.width + 0.5f) / lc_iH)) - GridLifeConstants.GOL_CELL_MIN_SIZE);
     }
    }
   }
  } 
  if (lc_iZoom < 0          ) lc_iZoom = 0          ;
  if (lc_iZoom > pa_iMaxZoom) lc_iZoom = pa_iMaxZoom;
  
  return lc_iZoom; 
 }
 
 
 
 /**
  * Scrolls to the selction's top left position
  * 
  */
 public final void scrollToSelectionForZoom()
 {
  int lc_iX1   = 0;
  int lc_iY1   = 0;
  int lc_iX2   = 0;
  int lc_iY2   = 0;
  int lc_iW    = 0;
  int lc_iH    = 0;
  int lc_iPx   = 0;
  int lc_iPy   = 0;
  
  lc_iX1 = correctLocation(Math.min(m_tCellSelectionPosTopLeft.x, m_tCellSelectionPosBottomRight.x), m_tProperties.getColumns());
  lc_iX2 = correctLocation(Math.max(m_tCellSelectionPosTopLeft.x, m_tCellSelectionPosBottomRight.x), m_tProperties.getColumns());
  lc_iY1 = correctLocation(Math.min(m_tCellSelectionPosTopLeft.y, m_tCellSelectionPosBottomRight.y), m_tProperties.getRows()   );
  lc_iY2 = correctLocation(Math.max(m_tCellSelectionPosTopLeft.y, m_tCellSelectionPosBottomRight.y), m_tProperties.getRows()   );
  lc_iW  = (lc_iX2 - lc_iX1); if (0 > lc_iW) lc_iW = GridLifeConstants.GOL_MIN_COLUMNS;
  lc_iH  = (lc_iY2 - lc_iY1); if (0 > lc_iH) lc_iH = GridLifeConstants.GOL_MIN_ROWS   ;  

  if (null != m_tScrollPane)
  {
   if (null != m_tScrollPane.getViewport())
   {
    lc_iPx = (m_iCellSize * lc_iX1); if (0 > lc_iPx) lc_iPx = 0;
    lc_iPy = (m_iCellSize * lc_iY1); if (0 > lc_iPy) lc_iPy = 0;
    
    scrollRectToVisible(new Rectangle(lc_iPx, lc_iPy, m_iCellSize *  lc_iW, m_iCellSize *  lc_iH)); 
   }
  }
 }
 
 
 
} // eoc
