/*
 * Game of Life
 * 
 */
package gridlife.ui;



/**
 * Imports
 * 
 */
import java.awt.event.AdjustmentListener ;



/**
 * Window for the main cell matrix.
 * 
 * @author Ulf Wagemann
 */
public class GridLifeCellMatrixWindow extends javax.swing.JInternalFrame {

  private GridLifeGuiConnector m_tGuiConnector;
  

  /**
   * Constructor
   * 
   * @param pa_tGuiConnector    GuiConnector object
   */
  public GridLifeCellMatrixWindow(GridLifeGuiConnector pa_tGuiConnector, javax.swing.event.DocumentListener pa_tDocumentListener) 
  {
   m_tGuiConnector = pa_tGuiConnector;
           
   initComponents();
   
   jTextFieldSlot.setDocument(new GridLifeQuickSlotDocument());
   jTextFieldSlot.getDocument().addDocumentListener(pa_tDocumentListener);
  }


  
  /**
   * Attaches the given adjustment listener to the cell matrix scrollbars
   * 
   * @param pa_tListener   Listener to attach
   */
  public final void setAdjustmentListener(AdjustmentListener pa_tAdjListener)
  {
   if (null != pa_tAdjListener)  
   {
    if (null != getVerticalScrollBar())
    {
     getVerticalScrollBar().addAdjustmentListener(pa_tAdjListener);
    }
    if (null != getHorizontalScrollBar())
    {
     getHorizontalScrollBar().addAdjustmentListener(pa_tAdjListener);
    }
   }
  }
  

  
  /**
   * Sets the zoom value to the given value 
   * 
   * @param pa_iValue 
   */
  public final void setZoomValue(int pa_iValue)
  {
   if (jSliderZoom.getMinimum() <= pa_iValue && jSliderZoom.getMaximum() >= pa_iValue)  
   {
    jSliderZoom.setValue(pa_iValue); 
   }
  }

  
  
  /**
   * Returns the maximum zoom value
   * 
   * @return   max zoom
   */
  public final int getMaxZoomValue() {return jSliderZoom.getMaximum();}
  
  
  
  /**
   * Sets the zoom value to the given value 1..10
   * 
   * @param pa_iValue 
   */
  public final void setFillRatio(int pa_iValue)
  {
   if (1 <= pa_iValue && 100 >= pa_iValue && null != jSpinnerPercent)  
   {
    jSpinnerPercent.setValue(pa_iValue);
   }
  }

  
  
  /**
   * Returns the fill ratio
   * 
   * @return  fill ratio, -1 in case of error
   */
  public final int getFillRatio()
  {
   Object lc_tValue  = null;
   String lc_sTemp   = null;
   int    lc_iResult = -1;
   
   if (null != jSpinnerPercent)
   {
    if (null != (lc_tValue = jSpinnerPercent.getValue()))
    {
     if (null != (lc_sTemp = lc_tValue.toString()))
     {
      lc_iResult = (new Integer(lc_sTemp)).intValue();
     }
    }
   }
   return lc_iResult;
  }
  
  
  
  /**
   * Returns whether the palette selection is active
   * 
   * @return  true, if the palette selection is active, false otherwise
   */
  public final boolean isShapePaletteSelectionActive()
  {
   return (jComboBoxPaintType.getSelectedIndex() == 0);  
  }
  
  

  /**
   * Returns whether the palette selection is active
   * 
   * @return  true, if the palette selection is active, false otherwise
   */
  public final boolean isPointSelectionActive()
  {
   return (jComboBoxPaintType.getSelectedIndex() == 1);  
  }

  
  
  /**
   * Returns whether the palette selection is active
   * 
   * @return  true, if the palette selection is active, false otherwise
   */
  public final boolean isCellMatrixSelectionActive()
  {
   return (jComboBoxPaintType.getSelectedIndex() == 2);  
  }
  
  
  /**
   * Returns whether the palette selection is active
   * 
   * @return  true, if the palette selection is active, false otherwise
   */
  public final boolean isPaletteEditorSelectionActive()
  {
   return (jComboBoxPaintType.getSelectedIndex() == 3);  
  }
  
  
  /**
   * Returns the paint mode
   * 
   * @return   paint mode
   */
  public final int getPaintMode() {return jComboBoxPaintMode.getSelectedIndex();}
  

  
  /**
   * Sets the paint mode
   * 
   * @return   new mode
   */
  public final void setPaintMode(int pa_iValue) {jComboBoxPaintMode.setSelectedIndex(pa_iValue);}
  

  
  
  /**
   * Returns the paint type
   * 
   * @return   paint type
   */
  public final int getPaintType() {return jComboBoxPaintType.getSelectedIndex();}
  
  
  /**
   * Sets the paint mode
   * 
   * @return   new mode
   */
  public final void setPaintType(int pa_iValue) {jComboBoxPaintType.setSelectedIndex(pa_iValue);}

  
  
  /**
   * Returns whether the XOR paint mode is active
   * 
   * @return  true, if the XOr mode is active, false otherwise
   */
  public final boolean isShapePaintXorModeActive()
  {
   return (jComboBoxPaintMode.getSelectedIndex() == 1);  
  }
  
  
  
  /**
   * Returns the zoom value
   * 
   * @return  zoom value 1..10, -1 in case of error
   */
  public final int getZoomValue() {return (null != jSliderZoom ? jSliderZoom.getValue() : -1);}
  
  
  
  /**
   * Returns the slot number
   * 
   * @return  "" in case of error, slot number otherwise
   */
      
  public final String getSlot() {return (null != jTextFieldSlot ? jTextFieldSlot.getText() : "");}
  
  
  
  /**
   * Sets the slot number
   * 
   * @param pa_iSlot  slot number 1.. 
   */
      
  public final void setSlot(String pa_sSlot) 
  {
   if (null != jTextFieldSlot && null != pa_sSlot)
   {
    jTextFieldSlot.setText(pa_sSlot); 
   }
  }
  
  
  
  /**
   * Adds the painting panel to the scroll pane
   * 
   * @param pa_tPanel    panel to add
   */
  public final void setCellPaintPanel(GridLifeCellMatrixPaintPanel pa_tPanel)
  {
   if (null != jScrollPaneCellMatrix) 
   {
    if (null != jScrollPaneCellMatrix.getViewport()) 
    {
     jScrollPaneCellMatrix.getViewport().add(pa_tPanel); 
    }
   }
  }
  

 
  /**
   * Sets the window title to the given value
   * 
   * @param pa_sTitle   Window title
   */
  public final void setWindowTitle(String pa_sTitle)
  {
   setTitle(pa_sTitle);
  }
  
  
  
  /**
   * Returns the window title, "" in case of empty title
   * 
   * @return  window title, can be ""
   */
  public final String getWindowTitle()
  {
   return (null != getTitle() ? getTitle() : ""); 
  }
  
  
  
  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the Form Editor.
   */
  @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        jScrollPaneCellMatrix = new javax.swing.JScrollPane();
        jSliderZoom = new javax.swing.JSlider();
        jPanelTop = new javax.swing.JPanel();
        jButtonStep = new javax.swing.JButton();
        jButtonPlay = new javax.swing.JButton();
        jButtonStop = new javax.swing.JButton();
        jButtonLoad = new javax.swing.JButton();
        jButtonSave = new javax.swing.JButton();
        jButtonClear = new javax.swing.JButton();
        jButtonFill = new javax.swing.JButton();
        jComboBoxPaintType = new javax.swing.JComboBox();
        jComboBoxPaintMode = new javax.swing.JComboBox();
        jSpinnerPercent = new javax.swing.JSpinner();
        jButtonFillWithDeadCells = new javax.swing.JButton();
        jButtonFillWithLivingCells = new javax.swing.JButton();
        jButtonPaste = new javax.swing.JButton();
        jButtonInvert = new javax.swing.JButton();
        jButtonUnset = new javax.swing.JButton();
        jButtonScreenshot = new javax.swing.JButton();
        jButtonZoomInto = new javax.swing.JButton();
        jButtonSelectAll = new javax.swing.JButton();
        jButtonSlotSave = new javax.swing.JButton();
        jButtonSlotLoad = new javax.swing.JButton();
        jTextFieldSlot = new javax.swing.JTextField();
        jButtonSlot = new javax.swing.JButton();
        filler2 = new javax.swing.Box.Filler(new java.awt.Dimension(0, 50), new java.awt.Dimension(0, 50), new java.awt.Dimension(32767, 50));
        filler3 = new javax.swing.Box.Filler(new java.awt.Dimension(0, 50), new java.awt.Dimension(0, 50), new java.awt.Dimension(32767, 50));
        filler4 = new javax.swing.Box.Filler(new java.awt.Dimension(0, 50), new java.awt.Dimension(0, 50), new java.awt.Dimension(32767, 50));
        jSeparator1 = new javax.swing.JSeparator();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setIconifiable(true);
        setMaximizable(true);
        setResizable(true);
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("gridlife/ui/resources/GridLifeCellMatrixWindow"); // NOI18N
        setTitle(bundle.getString("Form.title")); // NOI18N
        setDoubleBuffered(true);
        setName("Form"); // NOI18N

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(gridlife.core.GridLifeApp.class).getContext().getResourceMap(GridLifeCellMatrixWindow.class);
        jScrollPaneCellMatrix.setBackground(resourceMap.getColor("jScrollPaneCellMatrix.background")); // NOI18N
        jScrollPaneCellMatrix.setAlignmentX(1.0F);
        jScrollPaneCellMatrix.setAlignmentY(1.0F);
        jScrollPaneCellMatrix.setName("jScrollPaneCellMatrix"); // NOI18N

        jSliderZoom.setMajorTickSpacing(2);
        jSliderZoom.setMaximum(40);
        jSliderZoom.setMinorTickSpacing(1);
        jSliderZoom.setPaintLabels(true);
        jSliderZoom.setPaintTicks(true);
        jSliderZoom.setSnapToTicks(true);
        jSliderZoom.setValue(0);
        jSliderZoom.setName("jSliderZoom"); // NOI18N
        jSliderZoom.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                stateChangedSlider_CM_Zoom(evt);
            }
        });

        jPanelTop.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jPanelTop.setAlignmentX(0.0F);
        jPanelTop.setAlignmentY(1.0F);
        jPanelTop.setName("jPanelTop"); // NOI18N
        jPanelTop.setLayout(new java.awt.GridBagLayout());

        jButtonStep.setIcon(resourceMap.getIcon("jButtonStep.icon")); // NOI18N
        jButtonStep.setMnemonic(new Integer(java.awt.event.KeyEvent.VK_T));
        jButtonStep.setText(bundle.getString("jButtonStep.text")); // NOI18N
        jButtonStep.setToolTipText(bundle.getString("jButtonStep.tooltipText")); // NOI18N
        jButtonStep.setActionCommand(bundle.getString("jButtonStep.actionCommand")); // NOI18N
        jButtonStep.setMaximumSize(new java.awt.Dimension(30, 30));
        jButtonStep.setMinimumSize(new java.awt.Dimension(30, 30));
        jButtonStep.setName("jButtonStep"); // NOI18N
        jButtonStep.setPreferredSize(new java.awt.Dimension(30, 30));
        jButtonStep.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                actionPerformedButton_CM_Step(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 13;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 10, 0);
        jPanelTop.add(jButtonStep, gridBagConstraints);

        jButtonPlay.setIcon(resourceMap.getIcon("jButtonPlay.icon")); // NOI18N
        jButtonPlay.setText(bundle.getString("jButtonPlay.text")); // NOI18N
        jButtonPlay.setToolTipText(bundle.getString("jButtonPlay.tooltipText")); // NOI18N
        jButtonPlay.setActionCommand(bundle.getString("jButtonPlay.actionCommand")); // NOI18N
        jButtonPlay.setMaximumSize(new java.awt.Dimension(30, 30));
        jButtonPlay.setMinimumSize(new java.awt.Dimension(30, 30));
        jButtonPlay.setName("jButtonPlay"); // NOI18N
        jButtonPlay.setPreferredSize(new java.awt.Dimension(30, 30));
        jButtonPlay.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                actionPerformedButton_CM_Play(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 13;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 10, 0);
        jPanelTop.add(jButtonPlay, gridBagConstraints);

        jButtonStop.setIcon(resourceMap.getIcon("jButtonStop.icon")); // NOI18N
        jButtonStop.setText(bundle.getString("jButtonStop.text")); // NOI18N
        jButtonStop.setToolTipText(bundle.getString("jButtonStop.tooltipText")); // NOI18N
        jButtonStop.setActionCommand(bundle.getString("jButtonStop.actionCommand")); // NOI18N
        jButtonStop.setMaximumSize(new java.awt.Dimension(30, 30));
        jButtonStop.setMinimumSize(new java.awt.Dimension(30, 30));
        jButtonStop.setName("jButtonStop"); // NOI18N
        jButtonStop.setPreferredSize(new java.awt.Dimension(30, 30));
        jButtonStop.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                actionPerformedButton_CM_Stop(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 13;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 10, 10);
        jPanelTop.add(jButtonStop, gridBagConstraints);

        jButtonLoad.setIcon(resourceMap.getIcon("jButtonLoad.icon")); // NOI18N
        jButtonLoad.setText(bundle.getString("jButtonLoad.text")); // NOI18N
        jButtonLoad.setToolTipText(bundle.getString("jButtonLoad.tooltipText")); // NOI18N
        jButtonLoad.setActionCommand(bundle.getString("jButtonLoad.actionCommand")); // NOI18N
        jButtonLoad.setMaximumSize(new java.awt.Dimension(30, 30));
        jButtonLoad.setMinimumSize(new java.awt.Dimension(30, 30));
        jButtonLoad.setName("jButtonLoad"); // NOI18N
        jButtonLoad.setPreferredSize(new java.awt.Dimension(30, 30));
        jButtonLoad.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                actionPerformedButton_CM_LoadMatrix(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 8;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 0, 0);
        jPanelTop.add(jButtonLoad, gridBagConstraints);

        jButtonSave.setIcon(resourceMap.getIcon("jButtonSave.icon")); // NOI18N
        jButtonSave.setText(bundle.getString("jButtonSave.text")); // NOI18N
        jButtonSave.setToolTipText(bundle.getString("jButtonSave.tooltipText")); // NOI18N
        jButtonSave.setActionCommand(bundle.getString("jButtonSave.actionCommand")); // NOI18N
        jButtonSave.setMaximumSize(new java.awt.Dimension(30, 30));
        jButtonSave.setMinimumSize(new java.awt.Dimension(30, 30));
        jButtonSave.setName("jButtonSave"); // NOI18N
        jButtonSave.setPreferredSize(new java.awt.Dimension(30, 30));
        jButtonSave.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                actionPerformedButton_CM_SaveMatrix(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 8;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 0, 10);
        jPanelTop.add(jButtonSave, gridBagConstraints);

        jButtonClear.setIcon(resourceMap.getIcon("jButtonClear.icon")); // NOI18N
        jButtonClear.setText(bundle.getString("jButtonClear.text")); // NOI18N
        jButtonClear.setToolTipText(bundle.getString("jButtonClear.tooltipText")); // NOI18N
        jButtonClear.setActionCommand(bundle.getString("jButtonClear.actionCommand")); // NOI18N
        jButtonClear.setMaximumSize(new java.awt.Dimension(30, 30));
        jButtonClear.setMinimumSize(new java.awt.Dimension(30, 30));
        jButtonClear.setName("jButtonClear"); // NOI18N
        jButtonClear.setPreferredSize(new java.awt.Dimension(30, 30));
        jButtonClear.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                actionPerformedButton_CM_Clear(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 0, 0);
        jPanelTop.add(jButtonClear, gridBagConstraints);

        jButtonFill.setIcon(resourceMap.getIcon("jButtonFill.icon")); // NOI18N
        jButtonFill.setText(bundle.getString("jButtonFill.text")); // NOI18N
        jButtonFill.setToolTipText(bundle.getString("jButtonFill.tooltipText")); // NOI18N
        jButtonFill.setActionCommand(bundle.getString("jButtonFill.actionCommand")); // NOI18N
        jButtonFill.setMaximumSize(new java.awt.Dimension(30, 30));
        jButtonFill.setMinimumSize(new java.awt.Dimension(30, 30));
        jButtonFill.setName("jButtonFill"); // NOI18N
        jButtonFill.setPreferredSize(new java.awt.Dimension(30, 30));
        jButtonFill.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                actionPerformedButton_CM_Fill(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 10, 0, 0);
        jPanelTop.add(jButtonFill, gridBagConstraints);

        jComboBoxPaintType.setModel(new javax.swing.DefaultComboBoxModel(new String[] { bundle.getString("jComboBoxPaintType.itemPaletteSelection.text"), bundle.getString("jComboBoxPaintType.itemPointSelection.text"), bundle.getString("jComboBoxPaintType.itemCellSelection.text"), bundle.getString("jComboBoxPaintType.itemPaletteEditorSelection.text") }));
        jComboBoxPaintType.setSelectedItem(bundle.getString("jComboBoxPaintType.itemPaletteSelection.text"));
        jComboBoxPaintType.setMinimumSize(new java.awt.Dimension(100, 20));
        jComboBoxPaintType.setName("jComboBoxPaintType"); // NOI18N
        jComboBoxPaintType.setPreferredSize(new java.awt.Dimension(100, 20));
        jComboBoxPaintType.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                actionPerformedCombobox_CM_PaletteMode(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(10, 10, 0, 10);
        jPanelTop.add(jComboBoxPaintType, gridBagConstraints);

        jComboBoxPaintMode.setModel(new javax.swing.DefaultComboBoxModel(new String[] { bundle.getString("jComboBoxPaintMode.itemSet.text"), bundle.getString("jComboBoxPaintMode.itemXor.text") }));
        jComboBoxPaintMode.setSelectedItem(bundle.getString("jComboBoxPaintMode.itemSet.text"));
        jComboBoxPaintMode.setMinimumSize(new java.awt.Dimension(100, 20));
        jComboBoxPaintMode.setName("jComboBoxPaintMode"); // NOI18N
        jComboBoxPaintMode.setPreferredSize(new java.awt.Dimension(100, 20));
        jComboBoxPaintMode.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                actionPerformedCombobox_CM_ShapePaintmode(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 10, 0, 10);
        jPanelTop.add(jComboBoxPaintMode, gridBagConstraints);

        jSpinnerPercent.setModel(new javax.swing.SpinnerNumberModel(10, 1, 30, 1));
        jSpinnerPercent.setMinimumSize(new java.awt.Dimension(100, 20));
        jSpinnerPercent.setName("jSpinnerPercent"); // NOI18N
        jSpinnerPercent.setPreferredSize(new java.awt.Dimension(100, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 10);
        jPanelTop.add(jSpinnerPercent, gridBagConstraints);

        jButtonFillWithDeadCells.setIcon(resourceMap.getIcon("jButtonFillWithDeadCells.icon")); // NOI18N
        jButtonFillWithDeadCells.setText(bundle.getString("jButtonFillWithDeadCells.text")); // NOI18N
        jButtonFillWithDeadCells.setToolTipText(bundle.getString("jButtonFillWithDeadCells.tooltipText")); // NOI18N
        jButtonFillWithDeadCells.setActionCommand(bundle.getString("jButtonFillWithDeadCells.actionCommand")); // NOI18N
        jButtonFillWithDeadCells.setMaximumSize(new java.awt.Dimension(30, 30));
        jButtonFillWithDeadCells.setMinimumSize(new java.awt.Dimension(30, 30));
        jButtonFillWithDeadCells.setName("jButtonFillWithDeadCells"); // NOI18N
        jButtonFillWithDeadCells.setPreferredSize(new java.awt.Dimension(30, 30));
        jButtonFillWithDeadCells.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                actionPerformedButton_CM_FillWithDeadCells(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 10, 0, 0);
        jPanelTop.add(jButtonFillWithDeadCells, gridBagConstraints);

        jButtonFillWithLivingCells.setIcon(resourceMap.getIcon("jButtonFillWithLivingCells.icon")); // NOI18N
        jButtonFillWithLivingCells.setText(bundle.getString("jButtonFillWithLivingCells.text")); // NOI18N
        jButtonFillWithLivingCells.setToolTipText(bundle.getString("jButtonFillWithLivingCells.tooltipText")); // NOI18N
        jButtonFillWithLivingCells.setActionCommand(bundle.getString("jButtonFillWithLivingCells.actionCommand")); // NOI18N
        jButtonFillWithLivingCells.setMaximumSize(new java.awt.Dimension(30, 30));
        jButtonFillWithLivingCells.setMinimumSize(new java.awt.Dimension(30, 30));
        jButtonFillWithLivingCells.setName("jButtonFillWithLivingCells"); // NOI18N
        jButtonFillWithLivingCells.setPreferredSize(new java.awt.Dimension(30, 30));
        jButtonFillWithLivingCells.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                actionPerformedButton_CM_FillWithLivingCells(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 0, 0);
        jPanelTop.add(jButtonFillWithLivingCells, gridBagConstraints);

        jButtonPaste.setIcon(resourceMap.getIcon("jButtonPaste.icon")); // NOI18N
        jButtonPaste.setText(bundle.getString("jButtonPaste.text")); // NOI18N
        jButtonPaste.setToolTipText(bundle.getString("jButtonPaste.tooltipText")); // NOI18N
        jButtonPaste.setActionCommand(bundle.getString("jButtonPaste.actionCommand")); // NOI18N
        jButtonPaste.setMaximumSize(new java.awt.Dimension(30, 30));
        jButtonPaste.setMinimumSize(new java.awt.Dimension(30, 30));
        jButtonPaste.setName("jButtonPaste"); // NOI18N
        jButtonPaste.setPreferredSize(new java.awt.Dimension(30, 30));
        jButtonPaste.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                actionPerformedButton_CM_Paste(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 0, 0);
        jPanelTop.add(jButtonPaste, gridBagConstraints);

        jButtonInvert.setIcon(resourceMap.getIcon("jButtonInvert.icon")); // NOI18N
        jButtonInvert.setText(bundle.getString("jButtonInvert.text")); // NOI18N
        jButtonInvert.setToolTipText(bundle.getString("jButtonInvert.tooltipText")); // NOI18N
        jButtonInvert.setActionCommand(bundle.getString("jButtonInvert.actionCommand")); // NOI18N
        jButtonInvert.setMaximumSize(new java.awt.Dimension(30, 30));
        jButtonInvert.setMinimumSize(new java.awt.Dimension(30, 30));
        jButtonInvert.setName("jButtonInvert"); // NOI18N
        jButtonInvert.setPreferredSize(new java.awt.Dimension(30, 30));
        jButtonInvert.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                actionPerformedButton_CM_InvertCells(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 0, 10);
        jPanelTop.add(jButtonInvert, gridBagConstraints);

        jButtonUnset.setIcon(resourceMap.getIcon("jButtonUnset.icon")); // NOI18N
        jButtonUnset.setText(bundle.getString("jButtonUnset.text")); // NOI18N
        jButtonUnset.setToolTipText(bundle.getString("jButtonUnset.tooltipText")); // NOI18N
        jButtonUnset.setActionCommand(bundle.getString("jButtonUnset.actionCommand")); // NOI18N
        jButtonUnset.setMaximumSize(new java.awt.Dimension(30, 30));
        jButtonUnset.setMinimumSize(new java.awt.Dimension(30, 30));
        jButtonUnset.setName("jButtonUnset"); // NOI18N
        jButtonUnset.setPreferredSize(new java.awt.Dimension(30, 30));
        jButtonUnset.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                actionPerformedButton_CM_Unset(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 0, 10);
        jPanelTop.add(jButtonUnset, gridBagConstraints);

        jButtonScreenshot.setIcon(resourceMap.getIcon("jButtonScreenshot.icon")); // NOI18N
        jButtonScreenshot.setText(bundle.getString("jButtonScreenshot.text")); // NOI18N
        jButtonScreenshot.setToolTipText(bundle.getString("jButtonScreenshot.tooltipText")); // NOI18N
        jButtonScreenshot.setActionCommand(bundle.getString("jButtonScreenshot.actionCommand")); // NOI18N
        jButtonScreenshot.setMaximumSize(new java.awt.Dimension(30, 30));
        jButtonScreenshot.setMinimumSize(new java.awt.Dimension(30, 30));
        jButtonScreenshot.setName("jButtonScreenshot"); // NOI18N
        jButtonScreenshot.setPreferredSize(new java.awt.Dimension(30, 30));
        jButtonScreenshot.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                actionPerformedButton_CM_Screenshot(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 0, 10);
        jPanelTop.add(jButtonScreenshot, gridBagConstraints);

        jButtonZoomInto.setIcon(resourceMap.getIcon("jButtonZoomInto.icon")); // NOI18N
        jButtonZoomInto.setText(bundle.getString("jButtonZoomInto.text")); // NOI18N
        jButtonZoomInto.setToolTipText(bundle.getString("jButtonZoomInto.tooltipText")); // NOI18N
        jButtonZoomInto.setActionCommand(bundle.getString("jButtonZoomInto.actionCommand")); // NOI18N
        jButtonZoomInto.setMaximumSize(new java.awt.Dimension(30, 30));
        jButtonZoomInto.setMinimumSize(new java.awt.Dimension(30, 30));
        jButtonZoomInto.setName("jButtonZoomInto"); // NOI18N
        jButtonZoomInto.setPreferredSize(new java.awt.Dimension(30, 30));
        jButtonZoomInto.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonZoomIntoactionPerformedButton_CM_InvertCells(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 0, 10);
        jPanelTop.add(jButtonZoomInto, gridBagConstraints);

        jButtonSelectAll.setIcon(resourceMap.getIcon("jButtonSelectAll.icon")); // NOI18N
        jButtonSelectAll.setText(bundle.getString("jButtonSelectAll.text")); // NOI18N
        jButtonSelectAll.setToolTipText(bundle.getString("jButtonSelectAll.tooltipText")); // NOI18N
        jButtonSelectAll.setActionCommand(bundle.getString("jButtonSelectAll.actionCommand")); // NOI18N
        jButtonSelectAll.setMaximumSize(new java.awt.Dimension(30, 30));
        jButtonSelectAll.setMinimumSize(new java.awt.Dimension(30, 30));
        jButtonSelectAll.setName("jButtonSelectAll"); // NOI18N
        jButtonSelectAll.setPreferredSize(new java.awt.Dimension(30, 30));
        jButtonSelectAll.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                actionPerformedButton_CM_SelectAll(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 10, 0, 0);
        jPanelTop.add(jButtonSelectAll, gridBagConstraints);

        jButtonSlotSave.setIcon(resourceMap.getIcon("jButtonSlotSave.icon")); // NOI18N
        jButtonSlotSave.setText(bundle.getString("jButtonSlotSave.text")); // NOI18N
        jButtonSlotSave.setToolTipText(bundle.getString("jButtonSlotSave.tooltipText")); // NOI18N
        jButtonSlotSave.setActionCommand(bundle.getString("jButtonSave.actionCommand")); // NOI18N
        jButtonSlotSave.setMaximumSize(new java.awt.Dimension(30, 30));
        jButtonSlotSave.setMinimumSize(new java.awt.Dimension(30, 30));
        jButtonSlotSave.setName("jButtonSlotSave"); // NOI18N
        jButtonSlotSave.setPreferredSize(new java.awt.Dimension(30, 30));
        jButtonSlotSave.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                actionPerformedButton_CM_SlotSave(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 11;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 0, 10);
        jPanelTop.add(jButtonSlotSave, gridBagConstraints);

        jButtonSlotLoad.setIcon(resourceMap.getIcon("jButtonSlotLoad.icon")); // NOI18N
        jButtonSlotLoad.setText(bundle.getString("jButtonSlotLoad.text")); // NOI18N
        jButtonSlotLoad.setToolTipText(bundle.getString("jButtonSlotLoad.tooltipText")); // NOI18N
        jButtonSlotLoad.setActionCommand(bundle.getString("jButtonSave.actionCommand")); // NOI18N
        jButtonSlotLoad.setMaximumSize(new java.awt.Dimension(30, 30));
        jButtonSlotLoad.setMinimumSize(new java.awt.Dimension(30, 30));
        jButtonSlotLoad.setName("jButtonSlotLoad"); // NOI18N
        jButtonSlotLoad.setPreferredSize(new java.awt.Dimension(30, 30));
        jButtonSlotLoad.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                actionPerformedButton_CM_SlotLoad(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 11;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 0, 0);
        jPanelTop.add(jButtonSlotLoad, gridBagConstraints);

        jTextFieldSlot.setText(resourceMap.getString("jTextFieldSlot.text")); // NOI18N
        jTextFieldSlot.setMinimumSize(new java.awt.Dimension(100, 20));
        jTextFieldSlot.setName("jTextFieldSlot"); // NOI18N
        jTextFieldSlot.setPreferredSize(new java.awt.Dimension(100, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 10;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 10);
        jPanelTop.add(jTextFieldSlot, gridBagConstraints);

        jButtonSlot.setIcon(resourceMap.getIcon("jButtonSlot.icon")); // NOI18N
        jButtonSlot.setText(bundle.getString("jButtonSlot.text")); // NOI18N
        jButtonSlot.setToolTipText(bundle.getString("jButtonSlot.tooltipText")); // NOI18N
        jButtonSlot.setMaximumSize(new java.awt.Dimension(30, 30));
        jButtonSlot.setMinimumSize(new java.awt.Dimension(30, 30));
        jButtonSlot.setName("jButtonSlot"); // NOI18N
        jButtonSlot.setPreferredSize(new java.awt.Dimension(30, 30));
        jButtonSlot.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                actionPerformedButton_CM_Slot(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 11;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 10, 0, 0);
        jPanelTop.add(jButtonSlot, gridBagConstraints);

        filler2.setName("filler2"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.ipady = 30;
        gridBagConstraints.weightx = 1.0;
        jPanelTop.add(filler2, gridBagConstraints);

        filler3.setName("filler3"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 9;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.ipady = 30;
        gridBagConstraints.weightx = 1.0;
        jPanelTop.add(filler3, gridBagConstraints);

        filler4.setName("filler4"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 12;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanelTop.add(filler4, gridBagConstraints);

        jSeparator1.setName("jSeparator1"); // NOI18N
        jPanelTop.add(jSeparator1, new java.awt.GridBagConstraints());

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jSliderZoom, javax.swing.GroupLayout.DEFAULT_SIZE, 796, Short.MAX_VALUE)
                    .addComponent(jScrollPaneCellMatrix, javax.swing.GroupLayout.DEFAULT_SIZE, 796, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jPanelTop, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jPanelTop, javax.swing.GroupLayout.DEFAULT_SIZE, 1009, Short.MAX_VALUE)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addComponent(jScrollPaneCellMatrix, javax.swing.GroupLayout.DEFAULT_SIZE, 953, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(jSliderZoom, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

  
  /******************************************* getters for UI objects *****************************************************/
  
  public final javax.swing.JScrollBar getVerticalScrollBar()   {return jScrollPaneCellMatrix.getVerticalScrollBar();  }
  public final javax.swing.JScrollBar getHorizontalScrollBar() {return jScrollPaneCellMatrix.getHorizontalScrollBar();}
  
  public final javax.swing.JButton   getButtonSelectAll()   {return jButtonSelectAll       ;}
  public final javax.swing.JButton   getButtonClear()       {return jButtonClear           ;}
  public final javax.swing.JButton   getButtonFill()        {return jButtonFill            ;}
  public final javax.swing.JButton   getButtonLoad()        {return jButtonLoad            ;}
  public final javax.swing.JButton   getButtonPlay()        {return jButtonPlay            ;}
  public final javax.swing.JButton   getButtonSave()        {return jButtonSave            ;}
  public final javax.swing.JButton   getButtonStep()        {return jButtonStep            ;}
  public final javax.swing.JButton   getButtonStop()        {return jButtonStop            ;}
  public final javax.swing.JButton   getButtonZoomInto()    {return jButtonZoomInto        ;}
  public final javax.swing.JButton   getButtonScreenshot()  {return jButtonScreenshot      ;}
  public final javax.swing.JButton   getButtonSlot()        {return jButtonSlot            ;}
  public final javax.swing.JComboBox getComboBoxPalette()   {return jComboBoxPaintType   ;}
  public final javax.swing.JComboBox getComboBoxPaintMode() {return jComboBoxPaintMode;}

  public final javax.swing.JButton   getButtonFillWithDeadCells()   {return jButtonFillWithDeadCells;  }
  public final javax.swing.JButton   getButtonFillWithLivingCells() {return jButtonFillWithLivingCells;}
  public final javax.swing.JButton   getButtonInvert()              {return jButtonInvert;             } 
  public final javax.swing.JButton   getButtonPaste()               {return jButtonPaste;              }
  public final javax.swing.JButton   getButtonUnset()               {return jButtonUnset;              }
  public final javax.swing.JButton   getButtonSlotLoad()            {return jButtonSlotLoad;           }
  public final javax.swing.JButton   getButtonSlotSave()            {return jButtonSlotSave;           }
  
  public final javax.swing.JScrollPane getCellScrollPane() {return jScrollPaneCellMatrix   ;}
  public final javax.swing.JSlider     getSliderZoom()     {return jSliderZoom             ;}
  public final javax.swing.JSpinner    getSpinnerPercent() {return jSpinnerPercent         ;}
  
  public final javax.swing.JTextField  getTextFieldSlot() {return jTextFieldSlot;}

  /********************************************* event handling methods ****************************************************/  
  private void actionPerformedButton_CM_Fill(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_actionPerformedButton_CM_Fill
    if (null != m_tGuiConnector) m_tGuiConnector.handleButton_CM_Fill();
  }//GEN-LAST:event_actionPerformedButton_CM_Fill

  private void actionPerformedButton_CM_Clear(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_actionPerformedButton_CM_Clear
    if (null != m_tGuiConnector) m_tGuiConnector.handleButton_CM_Clear();
  }//GEN-LAST:event_actionPerformedButton_CM_Clear

  private void stateChangedSlider_CM_Zoom(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_stateChangedSlider_CM_Zoom
      if (null != m_tGuiConnector) m_tGuiConnector.handleSlider_CM_Zoom(evt);
  }//GEN-LAST:event_stateChangedSlider_CM_Zoom

  private void actionPerformedCombobox_CM_PaletteMode(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_actionPerformedCombobox_CM_PaletteMode
      if (null != m_tGuiConnector) m_tGuiConnector.handleCombobox_CM_PaletteMode();
  }//GEN-LAST:event_actionPerformedCombobox_CM_PaletteMode

  private void actionPerformedCombobox_CM_ShapePaintmode(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_actionPerformedCombobox_CM_ShapePaintmode
    if (null != m_tGuiConnector) m_tGuiConnector.handleCombobox_CM_ShapePaintMode();    
  }//GEN-LAST:event_actionPerformedCombobox_CM_ShapePaintmode

  private void actionPerformedButton_CM_SaveMatrix(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_actionPerformedButton_CM_SaveMatrix
    if (null != m_tGuiConnector) m_tGuiConnector.handleButton_CM_SaveMatrix();    
  }//GEN-LAST:event_actionPerformedButton_CM_SaveMatrix

  private void actionPerformedButton_CM_LoadMatrix(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_actionPerformedButton_CM_LoadMatrix
    if (null != m_tGuiConnector) m_tGuiConnector.handleButton_CM_LoadMatrix();    
  }//GEN-LAST:event_actionPerformedButton_CM_LoadMatrix

  private void actionPerformedButton_CM_Stop(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_actionPerformedButton_CM_Stop
    if (null != m_tGuiConnector) m_tGuiConnector.handleButton_CM_Stop();    
  }//GEN-LAST:event_actionPerformedButton_CM_Stop

  private void actionPerformedButton_CM_Play(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_actionPerformedButton_CM_Play
    if (null != m_tGuiConnector) m_tGuiConnector.handleButton_CM_Play();    
  }//GEN-LAST:event_actionPerformedButton_CM_Play

  private void actionPerformedButton_CM_Step(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_actionPerformedButton_CM_Step
    if (null != m_tGuiConnector) m_tGuiConnector.handleButton_CM_Step();    
  }//GEN-LAST:event_actionPerformedButton_CM_Step

private void actionPerformedButton_CM_Paste(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_actionPerformedButton_CM_Paste
    if (null != m_tGuiConnector) m_tGuiConnector.handleButton_CM_Paste();    
}//GEN-LAST:event_actionPerformedButton_CM_Paste

private void actionPerformedButton_CM_FillWithDeadCells(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_actionPerformedButton_CM_FillWithDeadCells
    if (null != m_tGuiConnector) m_tGuiConnector.handleButton_CM_FillWithDeadCells();      
}//GEN-LAST:event_actionPerformedButton_CM_FillWithDeadCells

private void actionPerformedButton_CM_FillWithLivingCells(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_actionPerformedButton_CM_FillWithLivingCells
    if (null != m_tGuiConnector) m_tGuiConnector.handleButton_CM_FillWithLivingCells();      
}//GEN-LAST:event_actionPerformedButton_CM_FillWithLivingCells

private void actionPerformedButton_CM_InvertCells(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_actionPerformedButton_CM_InvertCells
    if (null != m_tGuiConnector) m_tGuiConnector.handleButton_CM_InvertCells();      
}//GEN-LAST:event_actionPerformedButton_CM_InvertCells

private void actionPerformedButton_CM_Unset(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_actionPerformedButton_CM_Unset
    if (null != m_tGuiConnector) m_tGuiConnector.handleButton_CM_Unset();      
}//GEN-LAST:event_actionPerformedButton_CM_Unset

private void actionPerformedButton_CM_Screenshot(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_actionPerformedButton_CM_Screenshot
    if (null != m_tGuiConnector) m_tGuiConnector.handleButton_CM_Screenshot();      
}//GEN-LAST:event_actionPerformedButton_CM_Screenshot

private void jButtonZoomIntoactionPerformedButton_CM_InvertCells(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonZoomIntoactionPerformedButton_CM_InvertCells
    if (null != m_tGuiConnector) m_tGuiConnector.handleButton_CM_ZoomInto();      
}//GEN-LAST:event_jButtonZoomIntoactionPerformedButton_CM_InvertCells

private void actionPerformedButton_CM_SelectAll(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_actionPerformedButton_CM_SelectAll
    if (null != m_tGuiConnector) m_tGuiConnector.handleButton_CM_SelectAll();      
}//GEN-LAST:event_actionPerformedButton_CM_SelectAll

  private void actionPerformedButton_CM_SlotSave(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_actionPerformedButton_CM_SlotSave
    // TODO add your handling code here:
    if (null != m_tGuiConnector) m_tGuiConnector.handleButton_CM_SlotSave();      
  }//GEN-LAST:event_actionPerformedButton_CM_SlotSave

  private void actionPerformedButton_CM_SlotLoad(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_actionPerformedButton_CM_SlotLoad
    // TODO add your handling code here:
    if (null != m_tGuiConnector) m_tGuiConnector.handleButton_CM_SlotLoad();      
  }//GEN-LAST:event_actionPerformedButton_CM_SlotLoad

  private void actionPerformedButton_CM_Slot(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_actionPerformedButton_CM_Slot
    // TODO add your handling code here:
    if (null != m_tGuiConnector) m_tGuiConnector.handleButton_CM_SlotBrowse();      
  }//GEN-LAST:event_actionPerformedButton_CM_Slot

  
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.Box.Filler filler2;
    private javax.swing.Box.Filler filler3;
    private javax.swing.Box.Filler filler4;
    private javax.swing.JButton jButtonClear;
    private javax.swing.JButton jButtonFill;
    private javax.swing.JButton jButtonFillWithDeadCells;
    private javax.swing.JButton jButtonFillWithLivingCells;
    private javax.swing.JButton jButtonInvert;
    private javax.swing.JButton jButtonLoad;
    private javax.swing.JButton jButtonPaste;
    private javax.swing.JButton jButtonPlay;
    private javax.swing.JButton jButtonSave;
    private javax.swing.JButton jButtonScreenshot;
    private javax.swing.JButton jButtonSelectAll;
    private javax.swing.JButton jButtonSlot;
    private javax.swing.JButton jButtonSlotLoad;
    private javax.swing.JButton jButtonSlotSave;
    private javax.swing.JButton jButtonStep;
    private javax.swing.JButton jButtonStop;
    private javax.swing.JButton jButtonUnset;
    private javax.swing.JButton jButtonZoomInto;
    private javax.swing.JComboBox jComboBoxPaintMode;
    private javax.swing.JComboBox jComboBoxPaintType;
    private javax.swing.JPanel jPanelTop;
    public javax.swing.JScrollPane jScrollPaneCellMatrix;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JSlider jSliderZoom;
    private javax.swing.JSpinner jSpinnerPercent;
    private javax.swing.JTextField jTextFieldSlot;
    // End of variables declaration//GEN-END:variables
}
