/**
 * Game of Life
 * 
 */
package gridlife.ui;



/**
 * Imports
 * 
 */
import gridlife.color.GridLifeColorRecord;
import gridlife.core.GridLifeConstants;

import java.awt.Component            ;
import javax.swing.JList             ;
import javax.swing.JLabel            ;
import javax.swing.border.Border     ;
import javax.swing.border.EmptyBorder;
import javax.swing.ListCellRenderer  ;
import javax.swing.SwingConstants    ;



/**
 * Custom cell renderer for color list. The DefaultListCellRenderer class will take care of
 * the JLabels text property.
 * 
 * @author Ulf Wagemann
 */
public class GridLifeColorListCellRenderer extends JLabel
                                              implements ListCellRenderer 
{
 protected static  Border m_tNoFocusBorder;
 protected static  Border m_tFocusBorder  ; 
 
 
 /**
  * Constructor
  * 
  */
 public GridLifeColorListCellRenderer()
 { 
  super();
  
  m_tNoFocusBorder = new EmptyBorder(GridLifeConstants.GOL_COLOR_RENDERER_INSETS, GridLifeConstants.GOL_COLOR_RENDERER_INSETS, GridLifeConstants.GOL_COLOR_RENDERER_INSETS, GridLifeConstants.GOL_COLOR_RENDERER_INSETS);
  m_tFocusBorder   = new EmptyBorder(GridLifeConstants.GOL_COLOR_RENDERER_INSETS, GridLifeConstants.GOL_COLOR_RENDERER_INSETS, GridLifeConstants.GOL_COLOR_RENDERER_INSETS, GridLifeConstants.GOL_COLOR_RENDERER_INSETS);
  setOpaque(true);
  setBorder(m_tNoFocusBorder);
 }
 
 
 
 @Override
 public Component getListCellRendererComponent(JList   pa_tList        ,
                                               Object  pa_tValue       ,
                                               int     pa_iIndex       ,
                                               boolean pa_bIsSelected  ,
                                               boolean pa_bCellHasFocus) 
 {
  GridLifeColorRecord lc_tRecord = null;
  
  if (null != pa_tList)
  {
   setBackground(true == pa_bIsSelected ? pa_tList.getSelectionBackground() : pa_tList.getBackground());
   setForeground(true == pa_bIsSelected ? pa_tList.getSelectionForeground() : pa_tList.getForeground());
   setFont(pa_tList.getFont());
  
   setBorder(true == pa_bCellHasFocus ? m_tFocusBorder : m_tNoFocusBorder); // UIManager.getBorder("List.focusCellHighlightBorder")
   setHorizontalTextPosition(SwingConstants.RIGHT );
   setVerticalTextPosition  (SwingConstants.CENTER);
   setIconTextGap(GridLifeConstants.GOL_COLOR_RENDERER_GAP);
   
   if (pa_tValue instanceof GridLifeColorRecord)
   { 
    if (null != (lc_tRecord = (GridLifeColorRecord) pa_tValue))
    {
     setText(Integer.toString(lc_tRecord.getAge()));  
     setIcon(lc_tRecord.getImage());
    }
   }
   else
   {
    setText(pa_tValue.toString());
   }
  }
  return this;
 }

} // eoc




