/**
 * Game of Life
 * 
 */
package gridlife.ui;



/**
 * Imports
 * 
 */
import gridlife.color.GridLifeColorRecord;
import gridlife.color.GridLifeColorManager;



/**
 * Window for color palettes.
 * 
 * @author Ulf Wagemann
 */
public class GridLifeColorWindow extends javax.swing.JInternalFrame {

  private GridLifeGuiConnector m_tGuiConnector;

    
  /**
   * Constructor
   * 
   * @param pa_tGuiConnector    GuiConnector object
   */
  public GridLifeColorWindow(GridLifeGuiConnector pa_tGuiConnector) 
  {
   m_tGuiConnector = pa_tGuiConnector;
   initComponents();
  
  }
  
  

  /**
   * Updates the color sets 
   * 
   * @param pa_tColorSetManager   ColorSetManager
   * @param pa_sSetName           name of the active set
   */
  public final void updateColorSets(GridLifeColorManager pa_tColorSetManager, String pa_sSetName)
  {
   String[]    lc_tNames     = null;
   String      lc_sSet       = null;
   String      lc_sActiveSet = null;
   
   int lc_iCount = 0;
   int lc_iLen   = 0;
   int lc_iI     = 0;
   int lc_iActiveIndex = 0;
   
   if (null != pa_tColorSetManager)
   {
    if (null != (lc_tNames = pa_tColorSetManager.getColorSetNames()))
    {
     if (0 < (lc_iLen = lc_tNames.length))
     {
      for (lc_iI = 0; lc_iI < lc_iLen; lc_iI++)
      {
       if (null != (lc_sSet = lc_tNames[lc_iI]))
       {
        jComboBoxColorSets.addItem(lc_sSet); 
        
        if (null == pa_sSetName)
        {
         if (0 == lc_iI)
         {
          lc_sActiveSet = lc_sSet; 
         }
        }
        else
        {
         if (pa_sSetName.equals(lc_sSet)) 
         {
          lc_sActiveSet = lc_sSet; 
          lc_iActiveIndex = lc_iCount; 
         }
        }
       }
       lc_iCount++;
      } // for
      
     }
    }
   }
   if (0 < lc_iCount)
   {
    jComboBoxColorSets.setSelectedIndex(lc_iActiveIndex);
    if (null == pa_sSetName) 
    {
     pa_tColorSetManager.setActiveSetName(lc_sActiveSet);
    }
   }
  }

  
  
  /**
   * Returns the name of the selected palette. 
   * 
   * @return   Palette name
   */
  public final String getSelectedColorSetName()
  {
   return (String) jComboBoxColorSets.getSelectedItem();  
  }
  
  
  
  /**
   * Returns the name of the selected palette
   * 
   * @return   Palette name
   */
  public final boolean hasColorSetSelection()
  {
   return (jListColors.getSelectedIndex() != -1);
  }
  
  
  
  /**
   * Returns the name of the selected palette
   * 
   * @return   Palette name
   */
  public final int getColorSelectionIndex()
  {
   return (jListColors.getSelectedIndex());
  }
  
  
  /**
   * Returns the name of the selected palette
   * 
   * @return   Palette name
   */
  public final int getColorSelectionAge()
  {
   GridLifeColorRecord lc_tRecord = (GridLifeColorRecord) jListColors.getSelectedValue();
   
   return (null != lc_tRecord ? lc_tRecord.getAge() : 1);
  }
  

  
  
  /**
   * Sets the list renderer and initializes the list with data
   * 
   * @param pa_tData   List data
   */
  public final void initList(GridLifeColorRecord[] pa_tData, String pa_sSelection)
  {
   GridLifeColorListCellRenderer lc_tRenderer = new GridLifeColorListCellRenderer();   
   
   int     lc_iValue = (null != pa_sSelection ? Integer.valueOf(pa_sSelection) : 0);
   int     lc_iLen = 0;
   int     lc_iPos = 0;
   boolean lc_bFound = false;
   
   if (null != pa_tData)
   {
    jListColors.setBackground(java.awt.Color.white);
    jListColors.setListData(pa_tData);    
    jListColors.setCellRenderer(lc_tRenderer);  
    
    if (null == pa_sSelection)
    {
     jListColors.setSelectedIndex(0);
    }
    else
    {
     if (0 < (lc_iLen = pa_tData.length))
     {
      while (false == lc_bFound && lc_iPos < lc_iLen)
      {
       if (false == (lc_bFound = pa_tData[lc_iPos].getAge() == lc_iValue))
       {
        lc_iPos++; 
       }
      } // while
      
      jListColors.setSelectedIndex(false == lc_bFound ? 0 : lc_iPos);
     }
    }
   }
  }

  
  
  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the Form Editor.
   */
  @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPaneColorList = new javax.swing.JScrollPane();
        jListColors = new javax.swing.JList();
        jComboBoxColorSets = new javax.swing.JComboBox();

        setClosable(true);
        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setIconifiable(true);
        setResizable(true);
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("gridlife/ui/resources/GridLifeColorWindow"); // NOI18N
        setTitle(bundle.getString("Form.title")); // NOI18N
        setDoubleBuffered(true);
        setName("Form"); // NOI18N

        jScrollPaneColorList.setName("jScrollPaneColorList"); // NOI18N

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(gridlife.core.GridLifeApp.class).getContext().getResourceMap(GridLifeColorWindow.class);
        jListColors.setForeground(resourceMap.getColor("jListColors.foreground")); // NOI18N
        jListColors.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        jListColors.setName("jListColors"); // NOI18N
        jListColors.setVisibleRowCount(15);
        jListColors.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                valueChangedList_AC_Set(evt);
            }
        });
        jScrollPaneColorList.setViewportView(jListColors);

        jComboBoxColorSets.setName("jComboBoxColorSets"); // NOI18N
        jComboBoxColorSets.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                actionPerformedCombobox_AC_Sets(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jComboBoxColorSets, javax.swing.GroupLayout.Alignment.TRAILING, 0, 187, Short.MAX_VALUE)
                    .addComponent(jScrollPaneColorList, javax.swing.GroupLayout.DEFAULT_SIZE, 187, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jComboBoxColorSets, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPaneColorList, javax.swing.GroupLayout.DEFAULT_SIZE, 384, Short.MAX_VALUE)
                .addGap(19, 19, 19))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents
 
  
  /********************************************* event handling methods ****************************************************/
  private void actionPerformedCombobox_AC_Sets(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_actionPerformedCombobox_AC_Sets
     if (null != m_tGuiConnector) m_tGuiConnector.handleCombobox_AC_Sets(evt);  
  }//GEN-LAST:event_actionPerformedCombobox_AC_Sets

  private void valueChangedList_AC_Set(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_valueChangedList_AC_Set
     if (null != m_tGuiConnector) m_tGuiConnector.handleList_AC_ColorSet(evt);  
  }//GEN-LAST:event_valueChangedList_AC_Set

    
  
  /***************************************** getters for UI objects *****************************************************/
  public final javax.swing.JComboBox getComboboxColorSet() {return jComboBoxColorSets;}

  
  
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JComboBox jComboBoxColorSets;
    private javax.swing.JList jListColors;
    private javax.swing.JScrollPane jScrollPaneColorList;
    // End of variables declaration//GEN-END:variables
}
