/*
 * Game of Life
 * 
 */
package gridlife.ui;


/**
 * Interface for connecting the window class to the main class in order to divide logic and data from the representation, It also
 * enables other objects such as a computation threads or screenshot threads to invoke some methods of the main class :)
 * 
 * @author Ulf Wagemann
 */
public abstract interface GridLifeGuiConnector 
{
 /**
  * Menu main window
  */
 public abstract void handleMenuWindowProperties();
 
 public abstract void handleMenuWindowStatistics();
 
 public abstract void handleMenuWindowShapePalette();

 public abstract void handleMenuWindowPaletteEditor();
 
 public abstract void handleMenuAbout();
 
 public abstract void handleMenuExit();
 
 public abstract void handleMenuClear();
 
 public abstract void handleMenuLoad();
 
 public abstract void handleMenuSave();
 
 public abstract void handleMenuWindowColor();
 
 public abstract void handleMenuWindowArrange();
 
 public abstract void handleMenuCleanupScreenshots();
 
 public abstract void handleMenuCleanupQuickSaves();
 
 
 /**
  * Shape palette window
  * 
  */
 public abstract void handleCombobox_SP_Palette(java.awt.event.ActionEvent pa_tEvent);
 
 public abstract void handleList_SP_Palette(javax.swing.event.ListSelectionEvent pa_tEvent);
 
 
 
 /**
  * Buttons and stuff in cell window
  * 
  */
 public abstract void handleButton_CM_Fill();
 
 public abstract void handleButton_CM_Clear();
 
 public abstract void handleSlider_CM_Zoom(javax.swing.event.ChangeEvent pa_tEvent);
 
 public abstract void handleCombobox_CM_PaletteMode();
 
 public abstract void handleCombobox_CM_ShapePaintMode();
 
 public abstract void handleButton_CM_SaveMatrix();
 
 public abstract void handleButton_CM_LoadMatrix();
 
 public abstract void handleButton_CM_Step();
 
 public abstract void handleButton_CM_Stop();
 
 public abstract void handleButton_CM_Play();
 
 public abstract void handleButton_CM_FillWithDeadCells();
 
 public abstract void handleButton_CM_FillWithLivingCells();
 
 public abstract void handleButton_CM_InvertCells();
 
 public abstract void handleButton_CM_Paste();
 
 public abstract void handleButton_CM_Unset();
 
 public abstract void handleButton_CM_Screenshot();
 
 public abstract void handleButton_CM_ZoomInto();
 
 public abstract void handleButton_CM_SelectAll();
 
 public abstract void handleButton_CM_SlotSave();
 
 public abstract void handleButton_CM_SlotLoad();
 
 public abstract void handleButton_CM_SlotBrowse();
 
 
 
 /**
  * Properties window
  * 
  */
 public abstract void handleSpinner_PR_PropertyChanged();
 
 public abstract void handleButton_PR_Apply();
     
 public abstract void handleButton_PR_Reset();
 
 public abstract void handleButton_PR_SaveAs();
         
 public abstract void handleButton_PR_Load();

 public abstract void handleCombobox_PR_ShowHelpLines();
 
 public abstract void handleCombobox_PR_Topology();
 
 public abstract void handleCombobox_PR_Neighbourhood();
 
 public abstract void handleCombobox_PR_GridLines();
 
 public abstract void handleCombobox_PR_CellHiliteLines();
 
 public abstract void handleCombobox_PR_CellBorder();
  
 public abstract void handleCombobox_PR_InvertMouseWheel();
 
 public abstract void handleToggleButton_PR_Ruleset(java.awt.event.ItemEvent evt);
 
 public abstract void handleButton_PR_Color(int pa_iWhich, java.awt.Color pa_tOldColor);
 
 
 /**
  * Color window
  * 
  */
 public abstract void handleCombobox_AC_Sets(java.awt.event.ActionEvent pa_tEvent);
 
 public abstract void handleList_AC_ColorSet(javax.swing.event.ListSelectionEvent pa_tEvent);
 
 
 
 /**
  * Palette editor window
  * 
  */
 public abstract void handleButton_PE_InsertShapePalette();
 
 public abstract void handleButton_PE_Delete();
 
 public abstract void handleButton_PE_Clear();
 
 public abstract void handleButton_PE_InsertCellMatrix();
  
 public abstract void handleButton_PE_Load();
 
 public abstract void handleButton_PE_Save();
 
 public abstract void handleList_PE_Shapes(javax.swing.event.ListSelectionEvent pa_tEvent);
 
 
 
 /**
  * Quick slots dialog
  * 
  */
 public abstract void handleButton_QS_Close();
 
 
 
 /**
  * Program termination
  * 
  */
 public abstract void handleProgramExit();
 
 
 
 /**
  * Computation
  * 
  */
 public abstract void computeNextGeneration();
 
 
 
 /**
  * I/O threads
  * 
  */
 public abstract void notifyIOThreadStarted(String pa_sData);
 
 public abstract void notifyIOThreadTerminated(String pa_sData);
}
