/*
 * Game of Life
 * 
 */
package gridlife.ui;

import gridlife.core.GridLifeConstants;
import gridlife.core.GridLifeProperties;
import java.awt.Rectangle     ;
import java.awt.Color         ;
import java.awt.Graphics      ;
import java.awt.Graphics2D    ;
import java.awt.BasicStroke   ;
import java.awt.RenderingHints;
import java.awt.Composite     ;
import java.awt.AlphaComposite;
import javax.swing.JPanel     ;
import javax.swing.JScrollPane;


/**
 * Basic paint panel with global rendering stuff.
 * 
 * @author Ulf Wagemann
 */
public abstract class GridLifePaintPanel extends JPanel
{
 protected Color m_tColorBackground    ;
 protected Color m_tGridColor          ;
 protected Color m_tGridHelpLineColor  ;
 protected Color m_tCellBorderColor    ;
 protected Color m_tCellFillColor      ;
 protected Color m_tCellHiliteColor    ;
 protected Color m_tStatisticsLineColor;
 protected Color m_tCellMatrixInsertionFrameColor;
 protected Color m_tCellMatrixSelectionFrameColor;
 protected Color m_tCellMatrixSelectionAreaColor ;
 protected Color m_tCellMatrixInsertionAreaColor ;
 protected Color m_tShadowColor;
 
 protected static BasicStroke    m_tLineStrokeDotted;
 protected static BasicStroke    m_tLineStrokeSolid ;
 protected static RenderingHints m_tRenderingHints  ;
 protected static Composite      m_tAlphaComposite  ;
 
 protected JScrollPane        m_tScrollPane;
 protected GridLifeProperties m_tProperties;
 
 
  /**
   * Constructor
   * 
   */
 public GridLifePaintPanel(JScrollPane pa_tScrollPane, GridLifeProperties pa_tProperties)    
 {
  super();  
  setDoubleBuffered(true);
  
  m_tColorBackground     = new Color(GridLifeConstants.GOL_CELL_BACKGROUND_COLOR     );
  m_tGridColor           = new Color(GridLifeConstants.GOL_CELL_GRID_COLOR           );
  m_tCellBorderColor     = new Color(GridLifeConstants.GOL_CELL_BORDER_COLOR         );
  m_tCellFillColor       = new Color(GridLifeConstants.GOL_CELL_FILL_COLOR           );
  m_tCellHiliteColor     = new Color(GridLifeConstants.GOL_CELL_HILITE_COLOR         );
  m_tStatisticsLineColor = new Color(GridLifeConstants.GOL_CELL_STATISTICS_COLOR     );
  m_tGridHelpLineColor   = new Color(GridLifeConstants.GOL_CELL_GRID_HELPLINE_COLOR  );  
  m_tShadowColor         = new Color(GridLifeConstants.GOL_SHAPE_SHADOW1_COLOR       );
  
  m_tCellMatrixSelectionFrameColor = new Color(GridLifeConstants.GOL_CELL_MATRIX_SELECTION_BORDER_COLOR);
  m_tCellMatrixSelectionAreaColor  = new Color(GridLifeConstants.GOL_CELL_MATRIX_SELECTION_FILL_COLOR  );
  m_tCellMatrixInsertionAreaColor  = new Color(GridLifeConstants.GOL_CELL_MATRIX_INSERTION_AREA_COLOR  ); 
  m_tCellMatrixInsertionFrameColor = new Color(GridLifeConstants.GOL_CELL_MATRIX_INSERTION_FRAME_COLOR ); 
  
  m_tLineStrokeDotted  = new BasicStroke(1, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL, 1, new float[]{1}, 0);
  m_tLineStrokeSolid   = new BasicStroke(1);
  m_tRenderingHints    = new RenderingHints(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);  
  m_tAlphaComposite    = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, GridLifeConstants.GOL_CELL_MATRIX_SELECTION_ALPHA);
  
  m_tScrollPane        = pa_tScrollPane;
  m_tProperties        = pa_tProperties;
 }
 
 
 /**
  * Returns the properties object
  * 
  * @return  null in case of error, properties otherwise
  */
 public GridLifeProperties getProperties() {return m_tProperties;}
 
 
 
 /**
  * Sets the properties
  * 
  * @param pa_tProperties 
  */
 public void setProperties(GridLifeProperties pa_tProperties)
 {
  m_tProperties = pa_tProperties; 
 }
 
 
 
 /**
  * Does the custom painting on the given graphics object.
  * 
  * @param pa_tGraphics  Paint on it!
  */
 public abstract void paintCustom(Graphics2D pa_tGraphics);
 
 
 
/**
  * Overriden paintComponent() method which first lets the superclass perform its drawings and then
  * calls the custom drawing method.
  * 
  * @param pa_tGraphics Paint on it!
  */ 
 @Override public void paintComponent(Graphics pa_tGraphics)
 {
  super.paintComponent(pa_tGraphics);  
  
  Graphics2D lc_tGraphics2D = null;

  if (null != pa_tGraphics)
  {
   if (null != (lc_tGraphics2D = (Graphics2D) pa_tGraphics))
   {
    try
    {
     paintCustom(lc_tGraphics2D);  
    }
    catch (Exception lc_tException)
    {}
   }
  }
 }
 
   
} // eoc
