/**
 * Game of Life
 * 
 */
package gridlife.ui;



/**
 * Imports
 * 
 */
import java.util.Comparator ;
import java.util.Arrays     ;

import gridlife.shape.GridLifeShape   ;
import gridlife.shape.GridLifeShapeSet;



/**
 * Window for the shape palette editor
 * 
 * @author Ulf Wagemann
 */
public class GridLifePaletteEditorWindow extends javax.swing.JInternalFrame 
{
 private GridLifeGuiConnector m_tGuiConnector;


 
  /**
   * Constructor
   * 
   * @param pa_tGuiConnector    GuiConnector object
   */
  public GridLifePaletteEditorWindow(GridLifeGuiConnector pa_tGuiConnector)
  { 
   m_tGuiConnector = pa_tGuiConnector;
   initComponents();
      
   jListPaletteEditor.setCellRenderer(new GridLifeShapePaletteListCellRenderer()); 
   jListPaletteEditor.setModel(new GridLifeListModel());
   jListPaletteEditor.setBackground(java.awt.Color.white);
  }

  
  
  /**
   * Checks whether the list has a selection
   * 
   * @return   true, if the list has a selection, false otherwise
   */
  public final boolean hasListSelection() {return (-1 != jListPaletteEditor.getSelectedIndex());}
  

  
  /**
   * Checks whether the list has entries
   * 
   * @return   true, if the list has entries, false otherwise
   */
  public final boolean hasListEntries() {return (0 < jListPaletteEditor.getModel().getSize());}
  

  
  /**
   * Checks whether the list has an entry with the given name
   * 
   * @param pa_sName   name to check
   * @return   true, if the name already exists, false otherrwise
   */
  public final boolean hasEntry(String pa_sName)
  {
   GridLifeListModel  lc_tListModel = (GridLifeListModel) jListPaletteEditor.getModel(); 
   GridLifeShape           lc_tShape     = null;
   
   int lc_iSize = 0;
   int lc_iI    = 0;
   boolean lc_bFound = false;
   
   if (null != lc_tListModel && null != pa_sName)
   {
    if (false == pa_sName.isEmpty())
    {
     if (0 < (lc_iSize = lc_tListModel.getSize()))
     {
      while (false == lc_bFound && lc_iI < lc_iSize) 
      {
       if (null != (lc_tShape = (GridLifeShape) lc_tListModel.getElementAt(lc_iI)))
       {
        lc_bFound = (lc_tShape.getShapeName().equals(pa_sName));
       }
       lc_iI++;
      }
     }
    }
   }
   return lc_bFound;
  }
  
  
  
  /**
   * Returns the shapes
   * 
   * @return  shapes, null in case of error
   */
  public final GridLifeShapeSet getShapes()
  {
   GridLifeListModel     lc_tListModel = (GridLifeListModel) jListPaletteEditor.getModel(); 
   GridLifeShapeSet           lc_tShapes    =  null;
   GridLifeShape              lc_tShape     = null;        
   int lc_iSize = 0;
   int lc_iI    = 0;
   
   if (null != lc_tListModel)
   {
    if (0 < (lc_iSize = lc_tListModel.getSize()))
    {
     if (null != (lc_tShapes = new GridLifeShapeSet()))
     {
      for (lc_iI = 0; lc_iI < lc_iSize; lc_iI++)
      {
       if (null != (lc_tShape = (GridLifeShape) lc_tListModel.getElementAt(lc_iI)))
       {
        lc_tShapes.addShape(lc_tShape); 
       }
      } 
     }
    }
   }
   return lc_tShapes; 
  }
  
  
  
 
  
  /**
   * Clears the list and adds the shapes to it
   * 
   * @param pa_tShapeSet   shapeset
   */
  public final void initList(GridLifeShapeSet pa_tShapeSet)
  {
   GridLifeListModel  lc_tListModel = (GridLifeListModel) jListPaletteEditor.getModel();
   GridLifeShape[]         lc_tShapes    = null;
 
   int lc_iI      = 0;
   int lc_iLength = 0;
   
   if (null != lc_tListModel && null != pa_tShapeSet)
   {
    if (null != (lc_tShapes = pa_tShapeSet.getShapes()))
    {
     if (0 < (lc_iLength = lc_tShapes.length))
     {
      Arrays.sort(lc_tShapes, new Comparator<GridLifeShape>() 
                                   {
                                    public int compare(GridLifeShape o1, GridLifeShape o2)
                                    { 
                                     return o1.getShapeName().compareTo(o2.getShapeName());
                                    }
                                   }); 
     lc_tListModel.removeAllElements();
   
     for (lc_iI = 0; lc_iI < lc_iLength; lc_iI++)
     {
      lc_tListModel.addElement(lc_tShapes[lc_iI]); 
     }
     if (0 < lc_tListModel.getSize())
     {
      jListPaletteEditor.setSelectedIndex(0);
     }
    }
   }
  }
 }
  
  
  
  /**
   * Returns the list size
   * 
   * @return  0 in case of empty list, otherwise amount of members
   */
  public final int getListSize()
  {
   GridLifeListModel    lc_tListModel = (GridLifeListModel) jListPaletteEditor.getModel();
   return (null != lc_tListModel ? lc_tListModel.size() : 0);    
  }
  
  
  
  /**
   * Adds a shape to the list
   * 
   * param pa_tShape    shape
   */
  public final void addShapeToList(GridLifeShape pa_tShape)
  {
   javax.swing.DefaultListModel lc_tListModel = (javax.swing.DefaultListModel) jListPaletteEditor.getModel();
   
   lc_tListModel.addElement(pa_tShape);
   if (1 == lc_tListModel.getSize())
   {
    jListPaletteEditor.setSelectedIndex(0);
   }
  }
  
  
  
  /**
   * Deletes the currently selected shape from the list
   * 
   * @return true, if the list is empty now, false otherwise
   */
  public final boolean deleteSelectedShape()
  {
   int     lc_iPos   = jListPaletteEditor.getSelectedIndex();
   boolean lc_bEmpty = false;
   
   javax.swing.DefaultListModel lc_tListModel = (javax.swing.DefaultListModel) jListPaletteEditor.getModel();   
   lc_tListModel.removeElementAt(lc_iPos);
   lc_bEmpty = (0 == lc_tListModel.getSize());
     
   if (false == lc_bEmpty)
   {
    lc_iPos--; if (lc_iPos < 0) lc_iPos = 0; 
    jListPaletteEditor.setSelectedIndex(lc_iPos);
   }     
   return lc_bEmpty;
  }
  
  
  
  /**
   * Clears the shape list
   * 
   */
  public final void clearShapes()
  {
   javax.swing.DefaultListModel lc_tListModel = (javax.swing.DefaultListModel) jListPaletteEditor.getModel();   
   lc_tListModel.clear(); 
  }
  
  
  
  /**
   * Returns the currently selected shape
   * 
   * @return  null in case of empty palette or error, shape otherwise
   */
  public final GridLifeShape getSelectedShape()
  {
   Object        lc_tObject = null;
   GridLifeShape lc_tShape  = null;
   
   if (null != (lc_tObject =  jListPaletteEditor.getSelectedValue()))
   {
    if (lc_tObject instanceof GridLifeShape)
    {
     if (null != (lc_tShape = (GridLifeShape) lc_tObject))
     {
      return lc_tShape; 
     }
    }
   }
   return null;
  }
  
  
  
  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the Form Editor.
   */
  @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        jScrollPanePaletteEditor = new javax.swing.JScrollPane();
        jListPaletteEditor = new javax.swing.JList();
        jPanelButtons = new javax.swing.JPanel();
        jButtonDelete = new javax.swing.JButton();
        jButtonClear = new javax.swing.JButton();
        jButtonInsertMatrix = new javax.swing.JButton();
        jButtonInsertPalette = new javax.swing.JButton();
        jButtonLoad = new javax.swing.JButton();
        jButtonSave = new javax.swing.JButton();
        filler1 = new javax.swing.Box.Filler(new java.awt.Dimension(0, 0), new java.awt.Dimension(0, 0), new java.awt.Dimension(32767, 0));

        setClosable(true);
        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setIconifiable(true);
        setResizable(true);
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("gridlife/ui/resources/GridLifePaletteEditorWindow"); // NOI18N
        setTitle(bundle.getString("Form.title")); // NOI18N
        setDoubleBuffered(true);
        setName("Form"); // NOI18N
        getContentPane().setLayout(new java.awt.GridBagLayout());

        jScrollPanePaletteEditor.setName("jScrollPanePaletteEditor"); // NOI18N

        jListPaletteEditor.setModel(new javax.swing.DefaultListModel());
        jListPaletteEditor.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        jListPaletteEditor.setName("jListPaletteEditor"); // NOI18N
        jListPaletteEditor.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                valueChangedList_PE_Shapes(evt);
            }
        });
        jScrollPanePaletteEditor.setViewportView(jListPaletteEditor);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.ipadx = 253;
        gridBagConstraints.ipady = 355;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(10, 10, 8, 10);
        getContentPane().add(jScrollPanePaletteEditor, gridBagConstraints);

        jPanelButtons.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jPanelButtons.setAlignmentX(0.0F);
        jPanelButtons.setAlignmentY(0.0F);
        jPanelButtons.setName("jPanelButtons"); // NOI18N
        jPanelButtons.setLayout(new java.awt.GridBagLayout());

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(gridlife.core.GridLifeApp.class).getContext().getResourceMap(GridLifePaletteEditorWindow.class);
        jButtonDelete.setIcon(resourceMap.getIcon("jButtonDelete.icon")); // NOI18N
        jButtonDelete.setText(bundle.getString("jButtonDelete.text")); // NOI18N
        jButtonDelete.setToolTipText(bundle.getString("jButtonDelete.tooltipText")); // NOI18N
        jButtonDelete.setActionCommand(bundle.getString("jButtonDelete.actionCommand")); // NOI18N
        jButtonDelete.setMaximumSize(new java.awt.Dimension(30, 30));
        jButtonDelete.setMinimumSize(new java.awt.Dimension(30, 30));
        jButtonDelete.setName("jButtonDelete"); // NOI18N
        jButtonDelete.setPreferredSize(new java.awt.Dimension(30, 30));
        jButtonDelete.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                actionPerformedButton_PE_Delete(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 0);
        jPanelButtons.add(jButtonDelete, gridBagConstraints);

        jButtonClear.setIcon(resourceMap.getIcon("jButtonClear.icon")); // NOI18N
        jButtonClear.setText(bundle.getString("jButtonClear.text")); // NOI18N
        jButtonClear.setToolTipText(bundle.getString("jButtonClear.tooltipText")); // NOI18N
        jButtonClear.setActionCommand(bundle.getString("jButtonClear.actionCommand")); // NOI18N
        jButtonClear.setMaximumSize(new java.awt.Dimension(30, 30));
        jButtonClear.setMinimumSize(new java.awt.Dimension(30, 30));
        jButtonClear.setName("jButtonClear"); // NOI18N
        jButtonClear.setPreferredSize(new java.awt.Dimension(30, 30));
        jButtonClear.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                actionPerformedButton_PE_Clear(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(10, 5, 10, 0);
        jPanelButtons.add(jButtonClear, gridBagConstraints);

        jButtonInsertMatrix.setIcon(resourceMap.getIcon("jButtonInsertMatrix.icon")); // NOI18N
        jButtonInsertMatrix.setText(bundle.getString("jButtonInsertMatrix.text")); // NOI18N
        jButtonInsertMatrix.setToolTipText(bundle.getString("jButtonInsertMatrix.tooltipText")); // NOI18N
        jButtonInsertMatrix.setActionCommand(bundle.getString("jButtonInsertMatrix.actionCommand")); // NOI18N
        jButtonInsertMatrix.setMaximumSize(new java.awt.Dimension(30, 30));
        jButtonInsertMatrix.setMinimumSize(new java.awt.Dimension(30, 30));
        jButtonInsertMatrix.setName("jButtonInsertMatrix"); // NOI18N
        jButtonInsertMatrix.setPreferredSize(new java.awt.Dimension(30, 30));
        jButtonInsertMatrix.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                actionPerformedButton_PE_InsertCellMatrix(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 10, 0);
        jPanelButtons.add(jButtonInsertMatrix, gridBagConstraints);

        jButtonInsertPalette.setIcon(resourceMap.getIcon("jButtonInsertPalette.icon")); // NOI18N
        jButtonInsertPalette.setText(bundle.getString("jButtonInsertPalette.text")); // NOI18N
        jButtonInsertPalette.setToolTipText(bundle.getString("jButtonInsertPalette.tooltipText")); // NOI18N
        jButtonInsertPalette.setActionCommand(bundle.getString("jButtonInsertPalette.actionCommand")); // NOI18N
        jButtonInsertPalette.setMaximumSize(new java.awt.Dimension(30, 30));
        jButtonInsertPalette.setMinimumSize(new java.awt.Dimension(30, 30));
        jButtonInsertPalette.setName("jButtonInsertPalette"); // NOI18N
        jButtonInsertPalette.setPreferredSize(new java.awt.Dimension(30, 30));
        jButtonInsertPalette.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                actionPerformedButton_PE_InsertShapePalette(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(10, 5, 10, 0);
        jPanelButtons.add(jButtonInsertPalette, gridBagConstraints);

        jButtonLoad.setIcon(resourceMap.getIcon("jButtonLoad.icon")); // NOI18N
        jButtonLoad.setText(bundle.getString("jButtonLoad.text")); // NOI18N
        jButtonLoad.setToolTipText(bundle.getString("jButtonLoad.tooltipText")); // NOI18N
        jButtonLoad.setActionCommand(bundle.getString("jButtonLoad.actionCommand")); // NOI18N
        jButtonLoad.setMaximumSize(new java.awt.Dimension(30, 30));
        jButtonLoad.setMinimumSize(new java.awt.Dimension(30, 30));
        jButtonLoad.setName("jButtonLoad"); // NOI18N
        jButtonLoad.setPreferredSize(new java.awt.Dimension(30, 30));
        jButtonLoad.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                actionPerformedButton_PE_Load(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 5;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(10, 5, 10, 0);
        jPanelButtons.add(jButtonLoad, gridBagConstraints);

        jButtonSave.setIcon(resourceMap.getIcon("jButtonSave.icon")); // NOI18N
        jButtonSave.setText(bundle.getString("jButtonSave.text")); // NOI18N
        jButtonSave.setToolTipText(bundle.getString("jButtonSave.tooltipText")); // NOI18N
        jButtonSave.setActionCommand(bundle.getString("jButtonSave.actionCommand")); // NOI18N
        jButtonSave.setMaximumSize(new java.awt.Dimension(30, 30));
        jButtonSave.setMinimumSize(new java.awt.Dimension(30, 30));
        jButtonSave.setName("jButtonSave"); // NOI18N
        jButtonSave.setPreferredSize(new java.awt.Dimension(30, 30));
        jButtonSave.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                actionPerformedButton_PE_Save(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 6;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(10, 5, 10, 10);
        jPanelButtons.add(jButtonSave, gridBagConstraints);

        filler1.setName("filler1"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.weightx = 1.0;
        jPanelButtons.add(filler1, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.SOUTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 10, 10);
        getContentPane().add(jPanelButtons, gridBagConstraints);

        pack();
    }// </editor-fold>//GEN-END:initComponents

/***************************************** getters for UI objects *****************************************************/
public final javax.swing.JButton getButtonClear()         {return jButtonClear;            }
public final javax.swing.JButton getButtonDelete()        {return jButtonDelete;           }
public final javax.swing.JButton getButtonInsertMatrix()  {return jButtonInsertMatrix;     }  
public final javax.swing.JButton getButtonInsertPalette() {return jButtonInsertPalette;    }
public final javax.swing.JButton getButtonLoad()          {return jButtonLoad;             }  
public final javax.swing.JButton getButtonSave()          {return jButtonSave;             }
public final javax.swing.JScrollPane getScrollPaneEditor(){return jScrollPanePaletteEditor;}
  
  
  
/********************************************* event handling methods ****************************************************/  
private void actionPerformedButton_PE_Delete(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_actionPerformedButton_PE_Delete
if (null != m_tGuiConnector) m_tGuiConnector.handleButton_PE_Delete();      
}//GEN-LAST:event_actionPerformedButton_PE_Delete

private void actionPerformedButton_PE_InsertCellMatrix(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_actionPerformedButton_PE_InsertCellMatrix
if (null != m_tGuiConnector) m_tGuiConnector.handleButton_PE_InsertCellMatrix();      
}//GEN-LAST:event_actionPerformedButton_PE_InsertCellMatrix

private void actionPerformedButton_PE_InsertShapePalette(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_actionPerformedButton_PE_InsertShapePalette
if (null != m_tGuiConnector) m_tGuiConnector.handleButton_PE_InsertShapePalette();      
}//GEN-LAST:event_actionPerformedButton_PE_InsertShapePalette

private void actionPerformedButton_PE_Clear(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_actionPerformedButton_PE_Clear
if (null != m_tGuiConnector) m_tGuiConnector.handleButton_PE_Clear();        
}//GEN-LAST:event_actionPerformedButton_PE_Clear

private void actionPerformedButton_PE_Load(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_actionPerformedButton_PE_Load
if (null != m_tGuiConnector) m_tGuiConnector.handleButton_PE_Load();        
}//GEN-LAST:event_actionPerformedButton_PE_Load

private void actionPerformedButton_PE_Save(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_actionPerformedButton_PE_Save
if (null != m_tGuiConnector) m_tGuiConnector.handleButton_PE_Save();        
}//GEN-LAST:event_actionPerformedButton_PE_Save

private void valueChangedList_PE_Shapes(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_valueChangedList_PE_Shapes
if (null != m_tGuiConnector) m_tGuiConnector.handleList_PE_Shapes(evt);        
}//GEN-LAST:event_valueChangedList_PE_Shapes




    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.Box.Filler filler1;
    private javax.swing.JButton jButtonClear;
    private javax.swing.JButton jButtonDelete;
    private javax.swing.JButton jButtonInsertMatrix;
    private javax.swing.JButton jButtonInsertPalette;
    private javax.swing.JButton jButtonLoad;
    private javax.swing.JButton jButtonSave;
    private javax.swing.JList jListPaletteEditor;
    private javax.swing.JPanel jPanelButtons;
    private javax.swing.JScrollPane jScrollPanePaletteEditor;
    // End of variables declaration//GEN-END:variables
}
