/**
 * Game of Life
 * 
 */
package gridlife.ui;


/**
 * Imports
 * 
 */
import gridlife.core.GridLifeStatisticsRecord;



/**
 * Window for drawing statistics records.
 * 
 * @author Ulf Wagemann
 */
public class GridLifeStatisticsWindow extends javax.swing.JInternalFrame 
{
 private int m_iMaxCells;
 
 
  
  /**
   * Constructor
   * 
   * @param pa_tGuiConnector    GuiConnector object
   */
  public GridLifeStatisticsWindow() 
  {
   initComponents();
   m_iMaxCells = 0 ;
  }

  
  
  /**
   * Updates the display according to the statistics object
   * 
   * @param pa_tRecord   statistic object
   */
  public final void updateStatistics(GridLifeStatisticsRecord pa_tRecord)
  {
   double lc_dPercent    = 0.0 ;
   int    lc_iAmount     = 0   ;
   long   lc_lGeneration = 0   ;
   
   if (null != pa_tRecord)
   {
    lc_iAmount     = pa_tRecord.getCellAmount();
    lc_lGeneration = pa_tRecord.getGeneration();
    
    if (null != jTextFieldPercent)
    {
     jTextFieldPercent.setText(null);
     
     if (0 < m_iMaxCells)
     {
      lc_dPercent = (100.00 * lc_iAmount) / m_iMaxCells;
      jTextFieldPercent.setText(Double.toString(lc_dPercent));
      jTextFieldPercent.setCaretPosition(0);
     }
    }
    if (null != jTextFieldCells     ) {jTextFieldCells.setText     (Integer.toString(lc_iAmount    )); jTextFieldCells.setCaretPosition     (0);}
    if (null != jTextFieldGeneration) {jTextFieldGeneration.setText(Long.toString   (lc_lGeneration)); jTextFieldGeneration.setCaretPosition(0);}
    if (null != jTextFieldTendency  ) {jTextFieldTendency.setText  (pa_tRecord.getTendencyString()  ); jTextFieldTendency.setCaretPosition  (0);}
    
   }
  }
  
  
  /**
   * Adds the painting panel to the scroll pane
   * 
   * @param pa_tPanel    panel to add
   */
  public final void setStatisticsPaintPanel(GridLifeStatisticsPaintPanel pa_tPanel)
  {
   if (null != jScrollPanePopulationGraph) 
   {
    if (null != jScrollPanePopulationGraph.getViewport()) 
    {
     jScrollPanePopulationGraph.getViewport().add(pa_tPanel); 
    }
   }
  }
  
  
  
  
  /**
   * Adds the painting panel to the scroll pane
   * 
   * @param pa_tPanel    panel to add
   */
  public final void setHistogramPaintPanel(GridLifeAgeHistogramPaintPanel pa_tPanel)
  {
   if (null != jScrollPaneHistogram) 
   {
    if (null != jScrollPaneHistogram.getViewport()) 
    {
     jScrollPaneHistogram.getViewport().add(pa_tPanel); 
    }
   }
  }  
  
  
  
  /**
   * Displays the amount of running I/O threads
   * 
   * @param pa_sValue 
   */
  public final void setIOThreads(String pa_sValue)
  {
   jTextFieldIoThreads.setText(pa_sValue); 
  }
  

  /**
   * Sets the amount of max cells
   * 
   * @param pa_iValue   max cells
   */
  public final void setMaxCells(int pa_iValue)
  {
   m_iMaxCells = pa_iValue;
   jTextFieldMaxCells.setText(Integer.toString(pa_iValue)); 
  }
  
  
  
  /**
   * Displays the amount of used computing threads
   * 
   * @param pa_sValue 
   */
  public final void setComputingThreads(String pa_sValue)
  {
   jTextFieldComputingThreads.setText(pa_sValue); 
  }

  

 /***************************************** getters for UI objects *****************************************************/
 public javax.swing.JScrollPane getHistogramScrollPane() {return jScrollPaneHistogram;}
  
 public javax.swing.JScrollPane getStatisticsScrollPane() {return jScrollPanePopulationGraph;}
 
 
  
  
  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the Form Editor.
   */
  @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jLabelCells = new javax.swing.JLabel();
        jLabelGeneration = new javax.swing.JLabel();
        jLabelTendency = new javax.swing.JLabel();
        jTextFieldCells = new javax.swing.JTextField();
        jTextFieldGeneration = new javax.swing.JTextField();
        jTextFieldTendency = new javax.swing.JTextField();
        jScrollPanePopulationGraph = new javax.swing.JScrollPane();
        jTextFieldIoThreads = new javax.swing.JTextField();
        jLabelIo = new javax.swing.JLabel();
        jTextFieldComputingThreads = new javax.swing.JTextField();
        jLabelComputing = new javax.swing.JLabel();
        jScrollPaneHistogram = new javax.swing.JScrollPane();
        jLabelCells1 = new javax.swing.JLabel();
        jTextFieldMaxCells = new javax.swing.JTextField();
        jTextFieldPercent = new javax.swing.JTextField();
        jLabelPercent = new javax.swing.JLabel();

        setClosable(true);
        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setIconifiable(true);
        setResizable(true);
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("gridlife/ui/resources/GridLifeStatisticsWindow"); // NOI18N
        setTitle(bundle.getString("Form.title")); // NOI18N
        setDoubleBuffered(true);
        setName("Form"); // NOI18N

        jLabelCells.setLabelFor(jTextFieldCells);
        jLabelCells.setText(bundle.getString("jLabelCells.text")); // NOI18N
        jLabelCells.setName("jLabelCells"); // NOI18N

        jLabelGeneration.setLabelFor(jTextFieldGeneration);
        jLabelGeneration.setText(bundle.getString("jLabelGeneration.text")); // NOI18N
        jLabelGeneration.setName("jLabelGeneration"); // NOI18N

        jLabelTendency.setLabelFor(jTextFieldTendency);
        jLabelTendency.setText(bundle.getString("jLabelTendency.text")); // NOI18N
        jLabelTendency.setName("jLabelTendency"); // NOI18N

        jTextFieldCells.setEditable(false);
        jTextFieldCells.setText(bundle.getString("jTextFieldCells.text")); // NOI18N
        jTextFieldCells.setName("jTextFieldCells"); // NOI18N

        jTextFieldGeneration.setEditable(false);
        jTextFieldGeneration.setText(bundle.getString("jTextFieldGeneration.text")); // NOI18N
        jTextFieldGeneration.setName("jTextFieldGeneration"); // NOI18N

        jTextFieldTendency.setEditable(false);
        jTextFieldTendency.setText(bundle.getString("jTextFieldTendency.text")); // NOI18N
        jTextFieldTendency.setName("jTextFieldTendency"); // NOI18N

        jScrollPanePopulationGraph.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        jScrollPanePopulationGraph.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
        jScrollPanePopulationGraph.setName("jScrollPanePopulationGraph"); // NOI18N

        jTextFieldIoThreads.setEditable(false);
        jTextFieldIoThreads.setText(bundle.getString("jTextFieldIoThreads.text")); // NOI18N
        jTextFieldIoThreads.setName("jTextFieldIoThreads"); // NOI18N

        jLabelIo.setText(bundle.getString("jLabelIo.text")); // NOI18N
        jLabelIo.setName("jLabelIo"); // NOI18N

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(gridlife.core.GridLifeApp.class).getContext().getResourceMap(GridLifeStatisticsWindow.class);
        jTextFieldComputingThreads.setBackground(resourceMap.getColor("jTextFieldComputingThreads.background")); // NOI18N
        jTextFieldComputingThreads.setEditable(false);
        jTextFieldComputingThreads.setText(bundle.getString("jTextFieldComputingThreads.text")); // NOI18N
        jTextFieldComputingThreads.setName("jTextFieldComputingThreads"); // NOI18N

        jLabelComputing.setText(bundle.getString("jLabelComputing.text")); // NOI18N
        jLabelComputing.setName("jLabelComputing"); // NOI18N

        jScrollPaneHistogram.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        jScrollPaneHistogram.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
        jScrollPaneHistogram.setName("jScrollPaneHistogram"); // NOI18N
        jScrollPaneHistogram.setPreferredSize(new java.awt.Dimension(150, 200));

        jLabelCells1.setLabelFor(jTextFieldMaxCells);
        jLabelCells1.setText(bundle.getString("jLabelOf.text")); // NOI18N
        jLabelCells1.setName("jLabelOf"); // NOI18N

        jTextFieldMaxCells.setEditable(false);
        jTextFieldMaxCells.setText(bundle.getString("jTextFieldMaxCells.text")); // NOI18N
        jTextFieldMaxCells.setName("jTextFieldMaxCells"); // NOI18N

        jTextFieldPercent.setEditable(false);
        jTextFieldPercent.setText(bundle.getString("jTextFieldPercent.text")); // NOI18N
        jTextFieldPercent.setMaximumSize(new java.awt.Dimension(45, 20));
        jTextFieldPercent.setMinimumSize(new java.awt.Dimension(45, 20));
        jTextFieldPercent.setName("jTextFieldPercent"); // NOI18N
        jTextFieldPercent.setPreferredSize(new java.awt.Dimension(45, 20));
        jTextFieldPercent.setRequestFocusEnabled(false);

        jLabelPercent.setText(bundle.getString("jLabelPercent.text")); // NOI18N
        jLabelPercent.setName("jLabelPercent"); // NOI18N

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jLabelCells)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jTextFieldCells, javax.swing.GroupLayout.PREFERRED_SIZE, 58, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabelCells1)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jTextFieldMaxCells, javax.swing.GroupLayout.PREFERRED_SIZE, 73, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jTextFieldPercent, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabelPercent)
                        .addGap(30, 30, 30)
                        .addComponent(jLabelGeneration, javax.swing.GroupLayout.PREFERRED_SIZE, 63, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(2, 2, 2)
                        .addComponent(jTextFieldGeneration, javax.swing.GroupLayout.PREFERRED_SIZE, 63, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(jLabelTendency, javax.swing.GroupLayout.PREFERRED_SIZE, 56, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jTextFieldTendency, javax.swing.GroupLayout.PREFERRED_SIZE, 74, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 139, Short.MAX_VALUE)
                        .addComponent(jLabelComputing))
                    .addComponent(jScrollPanePopulationGraph, javax.swing.GroupLayout.DEFAULT_SIZE, 745, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jTextFieldComputingThreads, javax.swing.GroupLayout.PREFERRED_SIZE, 38, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(jLabelIo)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jTextFieldIoThreads, javax.swing.GroupLayout.PREFERRED_SIZE, 38, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(jScrollPaneHistogram, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );

        layout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {jTextFieldCells, jTextFieldMaxCells});

        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabelCells)
                    .addComponent(jTextFieldCells, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jTextFieldIoThreads, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabelIo)
                    .addComponent(jTextFieldComputingThreads, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabelComputing)
                    .addComponent(jLabelCells1)
                    .addComponent(jTextFieldMaxCells, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabelGeneration)
                    .addComponent(jTextFieldGeneration, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabelTendency)
                    .addComponent(jTextFieldTendency, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jTextFieldPercent, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabelPercent))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPanePopulationGraph, javax.swing.GroupLayout.DEFAULT_SIZE, 189, Short.MAX_VALUE)
                    .addComponent(jScrollPaneHistogram, javax.swing.GroupLayout.DEFAULT_SIZE, 189, Short.MAX_VALUE))
                .addContainerGap())
        );

        layout.linkSize(javax.swing.SwingConstants.VERTICAL, new java.awt.Component[] {jLabelCells, jLabelCells1, jLabelComputing, jLabelGeneration, jLabelIo, jLabelPercent, jLabelTendency});

        layout.linkSize(javax.swing.SwingConstants.VERTICAL, new java.awt.Component[] {jTextFieldCells, jTextFieldComputingThreads, jTextFieldGeneration, jTextFieldIoThreads, jTextFieldMaxCells, jTextFieldPercent, jTextFieldTendency});

        pack();
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabelCells;
    private javax.swing.JLabel jLabelCells1;
    private javax.swing.JLabel jLabelComputing;
    private javax.swing.JLabel jLabelGeneration;
    private javax.swing.JLabel jLabelIo;
    private javax.swing.JLabel jLabelPercent;
    private javax.swing.JLabel jLabelTendency;
    private javax.swing.JScrollPane jScrollPaneHistogram;
    private javax.swing.JScrollPane jScrollPanePopulationGraph;
    private javax.swing.JTextField jTextFieldCells;
    private javax.swing.JTextField jTextFieldComputingThreads;
    private javax.swing.JTextField jTextFieldGeneration;
    private javax.swing.JTextField jTextFieldIoThreads;
    private javax.swing.JTextField jTextFieldMaxCells;
    private javax.swing.JTextField jTextFieldPercent;
    private javax.swing.JTextField jTextFieldTendency;
    // End of variables declaration//GEN-END:variables
}
